
unit UDevice;
// Demo-Device Unit fr IPS-Module
// Thomas Dreler 2009
// frei fr private Nutzung

interface

uses Windows, SysUtils, Forms, StrUtils,  //standard units fr eigene Implementierung
     UIPSTypes, UIPSModuleTypes,UIPSDataTypes, //IPS SDK
     UModuleInterface; //eigenes Interface

type
//alle persistenten Status-Informationen werden mit get/set hier zugewiesen
  TSettings = record
        Logfile:String;
        echo:boolean;
  end;

//Hauptobjekt des Moduls, implementiert die folgenden Interfaces
TIPSDevice = class(TIPSModuleObject, //standard
                        IIPSModule, //standard
                        IIPSReceiveString, //TextEmpfang aus anderen Modulen ermglichen
                        IIPSDevice) //eigenes Interface

  private
   //--- Basic Structures
   fCurrentSettings: TSettings;
   fChangedSettings: TSettings;

   //--- Custom Objects
   lastLine:string;

   //--- get/set-Funktionen  (sind private)
   //alle GetSet-Funktionen fr Properties mssen stdcall benutzen!
   function getLogFileName:string; stdcall;
   procedure setLogFileName(FileName:string); stdcall;
   function getEcho:boolean;stdcall;
   procedure setEcho(bEcho:boolean);stdcall;

   //--- Private Procedures/Functions
   procedure logme(Text:string);
   procedure DoEcho(Text:string);
   procedure handleData(from,Text:String);

  protected
  {Alle Funktionen, die auch in abgeleiteten Objekten genutzt werden sollen,
  aber nicht ffentlich sind, werden als Protected gekennzeichnet
  Funktionen, die nicht implementiert werden, mssen auskommentiert werden

   Beispiel:
   IPS-funktionen zur optionalen Auswertung von Status-Informationen
   //Wenn eine Instance den Status ndert, kann hiermit drauf reagiert werden
   procedure ProcessInstanceStatusChange(InstanceID: TInstanceID; Status: Integer); override;

   //Wenn der Kernel den Status ndern mchte, kann ebenfalls noch reagiert werden
   procedure ProcessKernelRunlevelChange(Runlevel: Integer); override;
   }
  public
  {
  Alle nach Aussen sichtbare Funktionen und Werte mssen Public sein

  1. Objekterstellung , muss immer implementiert werden und public sein
  Dabei wird das Standardverhalten berschrieben (override)
  mit inherited kann die Standardfunktionalitt trotzdem noch genutzt werden
  }
  constructor Create(IKernel: IIPSKernel; InstanceID: TInstanceID); override;
   destructor  Destroy; override;

   //2. Class Functions
   class function GetModuleID(): TStrGUID; override;
   class function GetModuleType(): TIPSModuleType; override;
   class function GetModuleName(): String; override;
   class function GetImplemented(): TStrGUIDs; override;
   class function GetVendor(): String; override;
   class function GetAliases(): TStringArray; override;

   //3.optionale Funktionen
   class function GetParentRequirements(): TStrGUIDs; override;
   //class function GetClientRequirements(): TStrGUIDs; override;

   //4. Interface Funktionen
   //--- IIPSModule implementation
   procedure LoadSettings(); override;
   procedure SaveSettings(); override;
   procedure ResetChanges(); override;
   procedure ApplyChanges(); override;


   //--- IIPSReceiveString implemenatation(Datapoint)
   //alle Interface-Funktionen mssen stdcall benutzen!
   procedure ReceiveText(Text:string); stdcall;

  //--- IIPSDevice implementation =diese Funktionen sind unter PHP sichtbar
  {alle im Interface genannten Funktionen mssen Textgleich noch einmal
  hier aufgefhrt werden
  alle PHP-Funktionen mssen stdcall benutzen!
  }
  procedure setLogFile(FileName:string); stdcall;
  function setLine(Text:string):integer; stdcall;
  function getLine:string; stdcall;
  procedure EnableEcho(bEcho:boolean);stdcall;
  procedure Testfunction;stdcall;

 end;

implementation


//------------------------------------------------------------------------------
class function TIPSDevice.GetModuleType(): TIPSModuleType;
{
definiert den Modultyp.
Devices knnen keine Childs, nur Parents haben, Splitter dagegen beides.
Nur Devices werden bei der Modulauswahl angezeigt, wenn man nicht "AlleModuleAnzeigen"
ausgewhlt hat.
Regel:Wenn es ein Endgert ist, dann immer als Device, Wenn es dagegen Daten an andere
Module senden soll, als Splitter definieren
}
begin
 Result := mtDevice; //Typ ist i.d.R. mtDevice oder mtSplitter (siehe UIPSTypes)
end;
//------------------------------------------------------------------------------
class function TIPSDevice.GetModuleID(): TStrGUID;
//gibt die GUID des Moduls fr die Modulverwaltung zurck
begin
 Result := GUIDToString(IIPSDevice); //Will return Interface GUID
end;

//------------------------------------------------------------------------------
class function TIPSDevice.GetModuleName(): String;
{
gibt den Namen des Moduls fr die Modulverwaltung zurck
unter diesem Prefix z.B.die Eintrge im Log vorgenommen
}
begin
 Result := 'MyDemo';
end;

//------------------------------------------------------------------------------
class function TIPSDevice.GetParentRequirements(): TStrGUIDs;
{
abfrage, ob der Parent ein kompatibles Interface hat
Hier wollen wir nur Parents mit TextInterface Senden
Parents, die diese Vorraussetzung nicht haben, knnen nicht verbunden werden
}
begin
 SetLength(Result, 1);
 Result[0] := GUIDToString(IIPSSendString);

end;
//------------------------------------------------------------------------------
class function TIPSDevice.GetImplemented(): TStrGUIDs;

{

hier erfhrt der Parent mittels GetParentRequirement

(bzw der Child ber getChildRequirements),

ob er kompatibel mit diesem modul ist, sprich das gewnschte Interface implementiert ist

Hier sagen wir, das wir ber das TextInterface daten bekommen knnen

}

begin

 SetLength(Result, 1);
 Result[0] := GUIDToString(IIPSReceiveString);
end;

//------------------------------------------------------------------------------
class function TIPSDevice.GetVendor(): String;
//Das Modul erscheint unter diesem Herstellernamen in der Auswahl
begin
 Result := 'MySelf';
end;

//------------------------------------------------------------------------------
class function TIPSDevice.GetAliases(): TStringArray;
//Das Modul erscheint unter allen diesen Bamen in der Auswahl
begin

 SetLength(Result, 2); //Anzahl der Namen entsprechend anpassen
 Result[0] := 'MyDevice';
 Result[1] := 'MyLogger'; //optional

end;

//Ab hier gehts in eingemachte
//Implementation des Hauptobktes und der Interface
//------------------------------------------------------------------------------
constructor TIPSDevice.Create(IKernel: IIPSKernel; InstanceID: TInstanceID);

begin

 {StandardRoutinen zur Objekterzeugung eines IPS-Modules ausfhren
 }
 inherited;

 //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Create Object');
 {Properties definieren
 Diese Properties werden in die Settings bernammen. zu jer Property mssen
 die entsprechenden Get-/SetFunktionen definiert und erstellt werden
 Hier werden nur die Namen Get- und SetFunktionen ohne Parameter einegetragen.
 Eine Get-Funktion nimmt keine Parameter muss z.B. fr StrProperties einen String zurckliefern
 Die SetFunktion ist eigentlich eine Procedure wird ebenfalls ohne Parameter
 hier eingetragen, muss aber in der Implementierung einen Parameter mit dem geforderten Typ
 aufweisen. Der Name ist egal (siehe auch Delphi Hilfe Property)
 }
 RegisterStrProperty( 'LogFile',GetLogFileName,SetLogFileName);
 RegisterBoolProperty( 'Echo',GetEcho,SetEcho);
 //oder auch
 //RegisterIntProperty( 'IntVal',GetIntVal,SetIntVal);

 {Status-Variablen definieren (optional)
 Diese StatusVariablen werden dann in der Konsole und Frontend automatisch angezeigt
 Die StatusVariablen sollten auch ein profil bekommen, wenn sie in der Anzeige
 formatiert dargestellt werden sollen. Details bitte dem SDK (UIPSTypes ab
 Zeile 1925 fr SDK 2.04) entnehmen. Im WebFrontend sind in 2.0.4 aber nur die
 DefaultProfile implementiert, so das man bei eigenen Profile diese dort
 selber nachbilden muss.
 In 2.10 gibt man den Namen eines existierenden Defaultprofiles oder eines eigenen Profiles mit,
 welches man vorher mit CreateXXXXProfile angelegt hat. Bei eigenen Profilen kann man auch ein eigenes Icon
 angeben oder man whlt ein existierendes aus, wie hier.
 }
 TIPSVarProfile.CreateStringProfile('DemoTextProfile', 'Information','', '-->logged');
 RegisterVariable('LastLineVariable','LastText',vtString,'DemoTextProfile');
 RegisterVariable('TestVariable','Test',vtString,'DemoTextProfile');
 //Beispiel fr einen Schalter mit default Profile
 //RegisterVariable('StatusVariable','Status',vtBoolean,GetDefaultProfile(dpSwitch));

 //Initialisation lokaler variablen
 lastLine:='';

 //Hier werden auch Timer usw. initialisiert
 //...

 //Check Parent
 {hier wird automatisch die Verbindung zu einem passenden Parent hergestellt.
 steht der Parameter auf True, wird der Parent immer neu angelegt
 Die automatische Auswahl bei noch nicht registriertem Parent greift leider
 manchmal daneben, weil immer das erste passende Modul zugewiesen wird.
 auch wenn es schon woanders zugewiesen wurde.
 }
 RequireParent(IIPSSendString,false);

 //Load/Apply Settings
 ConstructorPostProcess;

 //Copy to save Settings even if Apply fails
 fCurrentSettings := fChangedSettings;

end;

//------------------------------------------------------------------------------
destructor  TIPSDevice.Destroy;
begin
 //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Destroy Object...');
 //Reset+Save Settings
 DestructorPreProcess;
 inherited;

end;

//--Implementation Klassenfunktionen
//------------------------------------------------------------------------------
procedure TIPSDevice.LoadSettings();
begin
{holt aus den gespeicherten Settings Properties und Variablen-Namen/Werte
Wurde diese vorher nicht mit Register... definiert, gibt es Fehler im Logfile
}

//Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Load Settings...');

 inherited;

 {hier kann man noch zustzliche Einstellungen  dazu laden.
 oder z.B. die wieder hergestellten gespeicherten Werte noch einmal
 bearbeiten, Status zurcksetzen.
 Die Ausfhrung von Aktionen ist noch nicht mglich
 }
 end;

//------------------------------------------------------------------------------
procedure TIPSDevice.SaveSettings();
begin
 {Speichert Properties und StrausVariablen in die Settings.xml}

//Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Save Settings...');
 inherited;
end;

//------------------------------------------------------------------------------
procedure TIPSDevice.ResetChanges();
{macht die im speicher vorhandenen nderungen wieder Rckgnig.
Dabei wird lediglich der Settings Record umkopiert. i.D.R arbeitet man jedoch
mit dem aktuellen (ungespeicherten) changedsettings record}
begin
 //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Reset Settings ...');

 fChangedSettings := fCurrentSettings;
 inherited;
end;

//------------------------------------------------------------------------------
procedure TIPSDevice.ApplyChanges();
{hier werden die StatusVariablen entgltig zugewiesen und am Ende ein
SaveSettings ausgefhrt}
begin

//Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Apply Changes');

 fCurrentSettings := fChangedSettings;
 inherited;
 {
 hier knnten jetzt noch Zusatzfunktionen zur Einstellung des Parents stehen
 ...

 Die Initialisierung des Moduls ist nun abgeschlossen.
 }
end;

// Implementation Get/Set-Funktionen
//------------------------------------------------------------------------------
//Get Funktion fr StringProperty Logfile
function TIPSDevice.getLogFileName:string;stdcall;
begin
    //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Read Property Logfile...');

    result:=fchangedsettings.Logfile;
end;
//------------------------------------------------------------------------------
//Set Funktion fr StringProperty Logfile
procedure TIPSDevice.setLogFileName(FileName:string);stdcall;
begin
    //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Set Property Logfile');

   if (FileName=fchangedSettings.Logfile) then
   begin
     exit; //nichts zu tun, wenn wir das Gleiche schon haben
   end;
    fchangedsettings.Logfile:=FileName; //ablegen

    {setzt ein Flag zum Speichern und
    erzeugt eine SettingsChanged-Message}
    settingschanged;
end;
//------------------------------------------------------------------------------

//Get Funktion fr BoolProperty echo
function TIPSDevice.getEcho;stdcall;
begin
    //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Read Property Echo...');
    result:=fchangedsettings.echo;
end;
//------------------------------------------------------------------------------
//Set Funktion fr BoolProperty Echo
procedure TIPSDevice.setEcho(bEcho:boolean);stdcall;
begin
    //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Set Property Echo');

   if (bEcho=fchangedSettings.echo) then
   begin
     exit; //nichts zu tun, wenn wir das Gleiche schon haben
   end;
    fchangedsettings.echo:=bEcho; //ablegen

    {setzt ein Flag zum Speichern und
    erzeugt eine SettingsChanged-Message}
    settingschanged;
end;

//Datapoint
{Implementation Dateninterface IIPSReceiveString
IIPSReceiveString erfordert die Implementierung einer Funktion ReceiveText
mit einem String als Parameter
}
//------------------------------------------------------------------------------
procedure TIPSDevice.ReceiveText(Text:string); stdcall;
begin

//Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'External Data received');

  //sendData erzeugt Eintrag im DebugFenster
  SendData('Received', ' Msg:'+Text);
  //Ausfhrung der Datenbearbeitung
  handleData('Interface',Text);
end;

//---Implementation IIPSDevice Modulfunctionen (Actions)
{Alle im DeviceInterface definierten Funktionen und Proceduren mssen auch
implementiert werden.
}
//------------------------------------------------------------------------------
//PHP-DemoFunktion zum setzen des Logfilenamens
procedure TIPSDevice.setLogFile(FileName:string);stdcall;
   begin
   //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Set Logfike entered');

   //sendData erzeugt Eintrag im DebugFenster
   SendData('SetLogfile', ' SetLogFile to '+FileName);
   //hier wird einfach nur die SetMethode aufgerufen
   setlogfilename(FileName);
   end;
 //------------------------------------------------------------------------------
 //PHP-DemoFunktion zur Rckgabe eines Wertes
 function TIPSDevice.getLine:string;stdcall;
 begin
 //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'GetLine entered...');
 //sendData erzeugt Eintrag im DebugFenster
 SendData('getLine','entered');
 //Als Ergenis wird der zuletzt gespeicherte Text zurckgegeben
 result:=lastLine;
 SendData('getLine','Finshed, Result:'+result);
end;

//------------------------------------------------------------------------------
//PHP-DemoFun zum ausfhren einer Aktion und Rckgabe eines Wertes
function TIPSDevice.setLine(Text:string):integer;stdcall;
begin
//Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'SetLine entered...');
   //sendData erzeugt Eintrag im DebugFenster
   senddata('setLine','entered');
   //Ausfhrung einer Logik-Funktion
   handleData('PHP',Text);
   //Rckgabe eines Wertes
   Result:=length(Text);
end;

//------------------------------------------------------------------------------
//---PHP Stub zum setzen einer Property
procedure TIPSDevice.EnableEcho(bEcho:boolean);stdcall;
begin
//Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Enable Echo entered...');
   //sendData erzeugt Eintrag im DebugFenster
   senddata('Enable Echo','entered');
   //Einfach nur die Set Methode aufrufen
  setEcho(bEcho);
end;

{ab hier kommt erst die Logik in ausgelagerten Funktionen
alternativ knnen auch aus anderen Programmen bereits vorhandene Funktionen in
eigenen Units verwendet werden (refactoring)
}
//---Implementation privater Modulfunctionen (Logik)
//------------------------------------------------------------------------------
procedure TIPSDevice.logMe(Text:string);
//implementiert Logger-Funktion->schreibt in eine Textdatei alle Aufrufe
var log:TextFile;
  fname:string;
begin
  //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'LogMe entered...');

 //sendData erzeugt Eintrag im DebugFenster
 senddata('LogMe','entered');
 //property Logfile abfragen
 fname:=GetLogFileName;
 //kein Name=nichts loggen
 if fname>'' then
 begin
  //fileffenen( Anhngen oder neu)
  assignfile(log,fname);
  if fileexists(fname) then append(log) else rewrite(log);
  //schreiben, dabei exceptions abfangen
  try
    writeln(log,Text);
  finally
  //file schliessen
  closefile(log);
  end;
 end;
 senddata('LogMe','finished');
end;
//------------------------------------------------------------------------------
procedure TIPSDevice.HandleData(From,Text:string);
//Verarbeitet die  ankommenden Daten in einer gemeinasmen Methode fr alle Quellen
var s:string;
begin
   //Nur zur Demo!! Eintrag im Kernellog zum Nachvollzienen der Aufrufe
 LogMessage(KL_DEBUG,'Handle Data entered');
  //sendData erzeugt Eintrag im DebugFenster
   senddata('HandleData','entered');

   //Demo-Funktion=Text zusammensetzen
   s:=from+' -->'+Text;
   lastline:=s;
   //Text loggen
   logme(s);
   //Echo zurckgeben, wenn gefordert
   DoEcho(Text);
   //Statusvariable setzen
   fKernel.VariableManager.WriteVariableString(GetStatusVariableID('LastLineVariable'), s);
   senddata('HandleData','finished');
end;
//------------------------------------------------------------------------------
procedure TIPSDevice.DoEcho(Text:string);
{
implementiert PseudoEcho-Funktion  gem Wunsch in
http://www.ip-symcon.de/forum/f13/beispielprojekt-doku-source-veroeffentlicht-7605/index2.html#post66603
->sendet den hereinkommenden Text an den angeschlossenen Parent zurck, soweit dieser das untersttzt.
}
var s:string;
  parent:IIPSModule;
  ifsend:IIPSSendString;
begin
  s:=Text;
  senddata('DoEcho','entered');
  //test, ob echo property gesetzt ist
  if getEcho() then
  begin
      senddata('DoEcho','Echo requested');
      //Parent Instance holen
      parent:=getParent();
      //Test ob Instance existiert
      if parent=NIL then
      begin
      //nein->Melden
        senddata('DoEcho','Kein Parent verbunden, skip');
      end else begin
      //test ob IIPSSendString-Interface existiert
        if supports(parent,IIPSSendString,IfSend) then
        begin
        //existiert, kann senden
          senddata('DoEcho','Alles Ok');
          ifsend.SendText(s);
        end else begin
        //Problem mitteilen
          senddata('DoEcho','Send String not supported');
        end;
      end;
   end else begin
   //kein Echo, weil nicht aktiviert, mache Debug-Eintrag
     senddata('DoEcho','Echo not requested');
   end;
   senddata('DoEcho','finished');
end;
procedure TIPSDevice.TestFunction; stdcall;
var
  i:integer;
  msg:string;
begin
  for I := 0 to 99 do
  begin
     senddata('TestFunction', 'Prepare String, Idx='+IntToStr(i));
     msg := msg + '1234567890';
     senddata('TestFunction', 'Write Variable, Idx='+IntToStr(i));
     fKernel.VariableManager.WriteVariableString(
        GetStatusVariableID('TestVariable'), msg);
  end;
end;
end.
