unit UIPSTypes;

interface

uses Windows,
     Messages,
     SysUtils,
     Classes,
     TypInfo,
     IniFiles,                                                     
     RTTIUnit,
     ActiveX,
     Math,
     Variants,
     InvokeRegistry,
     gnugettext,
     Contnrs,
     StrUtils,
     SOAPConst;


threadvar
 //This Value specifies if the created object should be DataContext aware
 DataContextDisabled: Boolean;

const
 // --- KERNEL VERSION FOR SDK
 KERNEL_VERSION   = {CompileVersion}$021E{/CompileVersion};

 // --- Important GUIDs
 IID_WildCard            : TGUID = '{FFFFFFFF-FFFF-FFFF-FFFF-FFFFFFFFFFFF}';
 IID_Blank               : TGUID = '{00000000-0000-0000-0000-000000000000}';

 // --- Message Constants
 // --- BASE MESSAGE
 IPS_BASE = 10000;                             //Base Message

 IPS_KERNELSHUTDOWN = IPS_BASE + 1;            //Pre Shutdown Message, Runlevel UNINIT Follows
 IPS_KERNELSTARTED = IPS_BASE + 2;             //Post Ready Message

 // --- KERNEL
 IPS_KERNELMESSAGE = IPS_BASE + 100;           //Kernel Message
 KR_CREATE = IPS_KERNELMESSAGE + 1;            //Kernel is beeing created
 KR_INIT = IPS_KERNELMESSAGE + 2;              //Kernel Components are beeing initialised, Modules loaded, Settings read
 KR_READY = IPS_KERNELMESSAGE + 3;             //Kernel is ready and running
 KR_UNINIT = IPS_KERNELMESSAGE + 4;            //Got Shutdown Message, unloading all stuff
 KR_SHUTDOWN = IPS_KERNELMESSAGE + 5;          //Uninit Complete, Destroying Kernel Inteface

 IPS_LOGMESSAGE = IPS_BASE + 200;              //Logmessage Message
 KL_MESSAGE = IPS_LOGMESSAGE + 1;              //Normal Message                      | FG: Black | BG: White  | STLYE : NONE
 KL_SUCCESS = IPS_LOGMESSAGE + 2;              //Success Message                     | FG: Black | BG: Green  | STYLE : NONE
 KL_NOTIFY = IPS_LOGMESSAGE + 3;               //Notiy about Changes                 | FG: Black | BG: Blue   | STLYE : NONE
 KL_WARNING = IPS_LOGMESSAGE + 4;              //Warnings                            | FG: Black | BG: Yellow | STLYE : NONE
 KL_ERROR = IPS_LOGMESSAGE + 5;                //Error Message                       | FG: Black | BG: Red    | STLYE : BOLD
 KL_DEBUG = IPS_LOGMESSAGE + 6;                //Debug Informations + Script Results | FG: Grey  | BG: White  | STLYE : NONE
 KL_CUSTOM = IPS_LOGMESSAGE + 7;               //User Message                        | FG: Black | BG: White  | STLYE : NONE

 // --- MODULE LOADER
 IPS_MODULEMESSAGE = IPS_BASE + 300;           //ModuleLoader Message
 ML_LOAD = IPS_MODULEMESSAGE + 1;              //Module loaded
 ML_UNLOAD = IPS_MODULEMESSAGE + 2;            //Module unloaded

 // --- OBJECT MANAGER
 IPS_OBJECTMESSAGE = IPS_BASE + 400;
 OM_REGISTER = IPS_OBJECTMESSAGE + 1;          //Object was registered
 OM_UNREGISTER = IPS_OBJECTMESSAGE + 2;        //Object was unregistered
 OM_CHANGEPARENT = IPS_OBJECTMESSAGE + 3;      //Parent was Changed
 OM_CHANGENAME = IPS_OBJECTMESSAGE + 4;        //Name was Changed
 OM_CHANGEINFO = IPS_OBJECTMESSAGE + 5;        //Info was Changed
 OM_CHANGETYPE = IPS_OBJECTMESSAGE + 6;        //Type was Changed
 OM_CHANGESUMMARY = IPS_OBJECTMESSAGE + 7;     //Summary was Changed
 OM_CHANGEPOSITION = IPS_OBJECTMESSAGE + 8;    //Position was Changed
 OM_CHANGEREADONLY = IPS_OBJECTMESSAGE + 9;    //ReadOnly was Changed
 OM_CHANGEHIDDEN = IPS_OBJECTMESSAGE + 10;     //Hidden was Changed
 OM_CHANGEICON = IPS_OBJECTMESSAGE + 11;       //Icon was Changed

 // --- INSTANCE MANAGER
 IPS_INSTANCEMESSAGE = IPS_BASE + 500;         //Instance Manager Message
 IM_CREATE = IPS_INSTANCEMESSAGE + 1;          //Instance created
 IM_DELETE = IPS_INSTANCEMESSAGE + 2;          //Instance deleted
 IM_CONNECT = IPS_INSTANCEMESSAGE + 3;         //Instance connectged
 IM_DISCONNECT = IPS_INSTANCEMESSAGE + 4;      //Instance disconncted
 IM_CHANGESTATUS = IPS_INSTANCEMESSAGE + 5;    //Status was Changed
 IM_CHANGESETTINGS = IPS_INSTANCEMESSAGE + 6;  //Settings were Changed
 IM_CHANGESEARCH = IPS_INSTANCEMESSAGE + 7;    //Searching was started/stopped
 IM_SEARCHUPDATE = IPS_INSTANCEMESSAGE + 8;    //Searching found new results
 IM_SEARCHPROGRESS = IPS_INSTANCEMESSAGE + 9;  //Searching progress in %
 IM_SEARCHCOMPLETE = IPS_INSTANCEMESSAGE + 10; //Searching is complete

 // --- VARIABLE MANAGER
 IPS_VARIABLEMESSAGE = IPS_BASE + 600;              //Variable Manager Message
 VM_CREATE = IPS_VARIABLEMESSAGE + 1;               //Variable Created
 VM_DELETE = IPS_VARIABLEMESSAGE + 2;               //Variable Deleted
 VM_UPDATE = IPS_VARIABLEMESSAGE + 3;               //On Variable Update
 VM_CHANGEPROFILENAME = IPS_VARIABLEMESSAGE + 4;    //On Profile Name Change
 VM_CHANGEPROFILEACTION = IPS_VARIABLEMESSAGE + 5;  //On Profile Action Change

 IPS_VARIABLEPROFILE = IPS_VARIABLEMESSAGE + 10;
 VP_CREATE = IPS_VARIABLEPROFILE + 1;
 VP_DELETE = IPS_VARIABLEPROFILE + 2;
 VP_CHANGETEXT = IPS_VARIABLEPROFILE + 3;
 VP_CHANGEVALUES = IPS_VARIABLEPROFILE + 4;
 VP_CHANGEDIGITS = IPS_VARIABLEPROFILE + 5;
 VP_CHANGEASSOCIATIONS = IPS_VARIABLEPROFILE + 6;
 VP_CHANGEICON = IPS_VARIABLEPROFILE + 7;

 // --- SCRIPT MANAGER
 IPS_SCRIPTMESSAGE = IPS_BASE + 700;           //Script Manager Message
 SM_CREATE = IPS_SCRIPTMESSAGE + 1;            //On Script Create
 SM_DELETE = IPS_SCRIPTMESSAGE + 2;            //On Script Delete
 SM_SETFILE = IPS_SCRIPTMESSAGE + 3;           //On Script SetFile
 SM_BROKEN = IPS_SCRIPTMESSAGE + 4;            //Script Broken Status changed

 // --- EVENT MANAGER
 IPS_EVENTMESSAGE = IPS_BASE + 800;             //Event Scripter Message
 EM_CREATE = IPS_EVENTMESSAGE + 1;             //On Event Create
 EM_DELETE = IPS_EVENTMESSAGE + 2;             //On Event Delete
 EM_CHANGE = IPS_EVENTMESSAGE + 3;             //On Event Change
 EM_UPDATED = IPS_EVENTMESSAGE + 4;

 // --- DATA HANDLER
 IPS_DATAMESSAGE = IPS_BASE + 900;             //Data Handler Message
 DM_CONNECT = IPS_DATAMESSAGE + 1;             //On Instance Connect
 DM_DISCONNECT = IPS_DATAMESSAGE + 2;          //On Instance Disconnect

 // --- SCRIPT ENGINE
 IPS_ENGINEMESSAGE = IPS_BASE + 1000;           //Script Engine Message
 SE_LIBUPDATE = IPS_ENGINEMESSAGE + 1;          //On Library Refresh
 SE_EXECUTED = IPS_ENGINEMESSAGE + 2;           //On Script Executed Delete
 SE_EXECUTING = IPS_ENGINEMESSAGE + 3;          //On Script Executing Start

 // --- TIMER POOL
 IPS_TIMERMESSAGE = IPS_BASE + 1100;            //Timer Pool Message
 TM_REGISTER = IPS_TIMERMESSAGE + 1;
 TM_UNREGISTER = IPS_TIMERMESSAGE + 2;
 TM_SETINTERVAL = IPS_TIMERMESSAGE + 3;
 TM_UPDATED = IPS_TIMERMESSAGE + 4;
 TM_EXECUTING = IPS_TIMERMESSAGE + 5;

 // --- MEDIA MANAGER
 IPS_MEDIAMESSAGE = IPS_BASE + 1200;           //Media Manager Message
 MM_CREATE = IPS_MEDIAMESSAGE + 1;             //On Media Create
 MM_DELETE = IPS_MEDIAMESSAGE + 2;             //On Media Delete
 MM_SETFILE = IPS_MEDIAMESSAGE + 3;            //On Media SetFile
 MM_AVAILABLE = IPS_MEDIAMESSAGE + 4;          //Media Available Status changed
 MM_UPDATED = IPS_MEDIAMESSAGE + 5;

 // --- LINK MANAGER
 IPS_LINKMESSAGE = IPS_BASE + 1300;           //Link Manager Message
 LM_CREATE = IPS_LINKMESSAGE + 1;             //On Link Create
 LM_DELETE = IPS_LINKMESSAGE + 2;             //On Link Delete
 LM_SETCHILD = IPS_LINKMESSAGE + 3;           //On Link

 // --- TInstanceStatus Constants

 // --- STATUS CODES
 IS_SBASE        = 100;
 IS_CREATING     = IS_SBASE + 1; //module is being created
 IS_ACTIVE       = IS_SBASE + 2; //module created and running
 IS_DELETING     = IS_SBASE + 3; //module us being deleted
 IS_INACTIVE     = IS_SBASE + 4; //module is not beeing used

 // --- ERROR CODES
 IS_EBASE        = 200;          //default errorcode
 IS_NOTCREATED   = IS_EBASE + 1; //instance could not be created

 // --- Search Handling
 FOUND_UNKNOWN = 0;     //Undefined value
 FOUND_NEW = 1;         //Device is new and not configured yet
 FOUND_OLD = 2;         //Device is already configues (InstanceID should be set)
 FOUND_CURRENT = 3;     //Device is already configues (InstanceID is from the current/searching Instance)
 FOUND_UNSUPPORTED = 4; //Device is not supported by Module

 type
// --- Basic Structures
 { ID Types }
 TIPSID = Word;
 TIPSIDs = Array of TIPSID;

 { Localization }
 TCategoryID = TIPSID;
 TCategoryIDs = Array of TCategoryID;
 TInstanceID = TIPSID;
 TInstanceIDs = Array of TInstanceID;
 TVariableID = TIPSID;
 TVariableIDs = Array of TVariableID;
 TScriptID = TIPSID;
 TScriptIDs = Array of TScriptID;
 TEventID = TIPSID;
 TEventIDs = Array of TEventID;
 TFormID = TIPSID;
 TFormIDs = Array of TFormID;
 TMediaID = TIPSID;
 TMediaIDs = Array of TMediaID;
 TLinkID = TIPSID;
 TLinkIDs = Array of TLinkID;
 TThreadID = Integer;
 TThreadIDs = Array of TThreadID;

 //TimerID's are not handled as IP-Symcon Objects
 TTimerID = Word;
 TTimerIDs = Array of TTimerID;

 TTimerEvent = procedure of object; stdcall;

 { Array of Variant's }
 TVariantArray = Array of Variant;

 { Array of String }
 TStringArray = Array of String;

 { String GUIDs }
 TStrGUID = String;
 TStrGUIDs = TStringArray;

 { Possible Object Types }
 TIPSObjectType = (otCategory, otInstance, otVariable, otScript, otEvent, otMedia, otLink);

 { Possible Module Types }
 TIPSModuleType = (mtCore, mtIO, mtSplitter, mtDevice, mtConfigurator);
 TModuleTypes = set of TIPSModuleType;

 //Forward declarations
 IIPSKernel = interface;
 IIPSModuleRegistry = interface;
 {$METHODINFO ON}
 IIPSModule = interface;
 IIPSSimpleKernel = interface;
 IIPSLicensePool = interface;
 IIPSModuleLoader = interface;
 IIPSObjectManager = interface;
 IIPSCategoryManager = interface;
 IIPSVariableManager = interface;
 IIPSInstanceManager = interface;
 IIPSScriptManager = interface;
 IIPSEventManager = interface;
 IIPSMediaManager = interface;
 IIPSLinkManager = interface;
 IIPSScriptEngine = interface;
 IIPSTimerPool = interface;
 IIPSFilePool = interface;
 IIPSDataHandler = interface;
 IIPSSOAPServer = interface;
 IIPSDebugServer = interface;
 {$METHODINFO OFF}
 IIPSInstanceManagerEx = interface;
 IIPSVariableManagerEx = interface;
 IIPSScriptEngineEx = interface;
 IIPSModuleLoaderEx = interface;
 IIPSTimerPoolEx = interface;
 IIPSFilePoolEx = interface;
 IIPSSettings = interface;

 TIPSModuleObject = class;
 TIPSModuleClass = class of TIPSModuleObject;

 TIPSCallableObject = class;
 TIPSCallableClass = class of TIPSCallableObject;

//------------------------------------------------------------------------------
// Basic Types
//------------------------------------------------------------------------------
 TIPSRemotable = class(TRemotable)
  public
   constructor Create; override;
 end;

//------------------------------------------------------------------------------
// Message Handling
//------------------------------------------------------------------------------
 { Message Record }
 TIPSMessage = class(TIPSRemotable)
  private
   FTimeStamp : TDateTime;
   FSenderID  : TIPSID;
   FMessage   : Integer;
   FData      : Variant;
  public
   constructor Create(SenderID: TIPSID; Message: Integer; Data: Array of Const); reintroduce; overload;
   constructor Create(Msg: TIPSMessage); reintroduce; overload;
  published
   property TimeStamp : TDateTime read FTimeStamp;
   property SenderID  : TIPSID read FSenderID;
   property Message   : Integer read FMessage;
   property Data      : Variant read FData write FData;
 end;
 TIPSMessages = Array of TIPSMessage;

//------------------------------------------------------------------------------
// Module DLL Exports
//------------------------------------------------------------------------------
 { Module Register Call }
 TFModuleRegister    = procedure(Kernel: IIPSKernel; ModuleRegistry: IIPSModuleRegistry); stdcall;
 { Module Unregister Call }
 TFModuleUnregister  = procedure(); stdcall;

//------------------------------------------------------------------------------
// Module Handling
//------------------------------------------------------------------------------
 PIPSLibraryInfo = ^TIPSLibraryInfo;
 TIPSLibraryInfo = record
  mUniqueID      : TStrGUID;
  mAuthor        : String;
  mURL           : String;
  mName          : String;
  mVersion       : Word;                                                        { HiByte - MajorV, LoByte - MinorV }
  mBuild         : Word;                                                        { 0 - 65535 }
  mDate          : TDateTime;                                                   { Build Date }
  //--------------------------------
  { Defines the Kernel Version the Module was compiled for }
  mKernelVersion : Integer;
  //--------------------------------
  fRegister      : TFModuleRegister;
  fUnregister    : TFModuleUnregister;
 end;

 { Must be EXPORTED by a Module }
 TFLibraryInfo = procedure(var LibraryInfo: PIPSLibraryInfo); stdcall;

 TIPSLibraryInformation = class(TIPSRemotable)
  private
   FLibraryID : TStrGUID;
   FAuthor   : String;
   FURL      : String;
   FName     : String;
   FVersion  : Word;
   FBuild    : Word;
   FDate     : TDateTime;
  published
   property LibraryID : TStrGUID read FLibraryID write FLibraryID;
   property Author    : String read FAuthor write FAuthor;
   property URL       : String read FURL write FURL;
   property Name      : String read FName write FName;
   property Version   : Word read FVersion write FVersion;
   property Build     : Word read FBuild write FBuild;
   property Date      : TDateTime read FDate write FDate;
 end;
 TIPSLibraryInformations = Array of TIPSLibraryInformation;

 TIPSBasicModuleInformation = class(TIPSRemotable)
  private
  { Used to requery/ident parent }
   FModuleID    : TStrGUID;
  { Used to show some userinfo while module isn't loaded but involved }
   FModuleName  : ShortString;
  { Possible Module Types }
   FModuleType  : TIPSModuleType;
  published
   property ModuleID   : TStrGUID read FModuleID write FModuleID;
   property ModuleName : ShortString read FModuleName write FModuleName;
   property ModuleType : TIPSModuleType read FModuleType write FModuleType;
 end;
 TIPSBasicModuleInformations = Array of TIPSBasicModuleInformation;


 TIPSModuleInformation = class(TIPSBasicModuleInformation)
  private
  { Used to requery/ident parent }
   FModuleID    : TStrGUID;
  { Used to show some userinfo while module isn't loaded but involved }
   FModuleName  : ShortString;
  { Possible Module Types }
   FModuleType  : TIPSModuleType;
  { More Information }
   FParentRequirements : TStrGUIDs;
   FChildRequirements  : TStrGUIDs;
   FImplemented : TStrGUIDs;
   FVendor      : String;
   FAliases     : TStringArray;
   { Linked Library }
   FLibraryID   : TStrGUID;
  published
   property ModuleID   : TStrGUID read FModuleID write FModuleID;
   property ModuleName : ShortString read FModuleName write FModuleName;
   property ModuleType : TIPSModuleType read FModuleType write FModuleType;
   property ParentRequirements : TStrGUIDs read FParentRequirements write FParentRequirements;
   property ChildRequirements : TStrGUIDs read FChildRequirements write FChildRequirements;
   property Implemented : TStrGUIDs read FImplemented write FImplemented;
   property Vendor : String read FVendor write FVendor;
   property Aliases : TStringArray read FAliases write FAliases;
   property LibraryID : TStrGUID read FLibraryID write FLibraryID;
 end;
 TIPSModuleInformations = Array of TIPSModuleInformation;

//------------------------------------------------------------------------------
// Object Handling
//------------------------------------------------------------------------------
 TIPSObject = class(TIPSRemotable)
  private
   FParentID       : TIPSID;
   FObjectID       : TIPSID;
   FObjectType     : TIPSObjectType;
   FObjectName     : String;
   FObjectInfo     : String;
   FObjectIcon     : String;
   FObjectSummary  : String; //Will not be saved!
   FObjectPosition : Integer;

   FObjectIsReadOnly : Boolean;
   FObjectIsHidden   : Boolean;

   //Special Fields for faster SOAP Access
   FIsPersistent   : Boolean;
   FHasChildren    : Boolean;
   FChildrenIDs    : TIPSIDs;
  published
   property ParentID         : TIPSID read FParentID write FParentID;
   property ObjectID         : TIPSID read FObjectID write FObjectID;
   property ObjectType       : TIPSObjectType read FObjectType write FObjectType;
   property ObjectName       : String read FObjectName write FObjectName;
   property ObjectInfo       : String read FObjectInfo write FObjectInfo;
   property ObjectIcon       : String read FObjectIcon write FObjectIcon;
   property ObjectSummary    : String read FObjectSummary write FObjectSummary;
   property ObjectPosition   : Integer read FObjectPosition write FObjectPosition;
   property ObjectIsReadOnly : Boolean read FObjectIsReadOnly write FObjectIsReadOnly;
   property ObjectIsHidden   : Boolean read FObjectIsHidden write FObjectIsHidden;
   //Special Properties for faster SOAP Access
   property IsPersistent     : Boolean read FIsPersistent write FIsPersistent;
   property HasChildren      : Boolean read FHasChildren write FHasChildren;
   property ChildrenIDs      : TIPSIDs read FChildrenIDs write FChildrenIDs;
 end;
 TIPSObjects = Array of TIPSObject;

//------------------------------------------------------------------------------
// Category Handling
//------------------------------------------------------------------------------
 TIPSCategory = class(TIPSRemotable)
  private
   FCategoryID         : TIPSID;
   FCategoryBackground : TMediaID;
  published
   property CategoryID         : TIPSID read FCategoryID write FCategoryID;
   property CategoryBackground : TMediaID read FCategoryBackground write FCategoryBackground;
 end;
 TIPSCategorys = Array of TIPSCategory;

//------------------------------------------------------------------------------
// Instance Handling
//------------------------------------------------------------------------------
 TIPSInterfaceStatus = Integer;

 TIPSInstance = class(TIPSRemotable)
  private
   { Identify the Instance }
   FInstanceID        : TInstanceID;
   { ParentID - Result of DataHandler.GetParentID(InstanceID) - Optimized for SOAP Access}
   FParentID          : TInstanceID;
   { ChildrenIDs - Result of DataHandler.GetChildrenInstanceIDs(InstanceID) - Optimized for SOAP Access}
   FChildrenIDs       : TInstanceIDs;
   { Parent needed? - Result of Length(ModuleInforation.ParentRequirements) > 0 - Optimized for SOAP Access }
   FNeedParent        : Boolean;
   { Interface instance of a Module }
   FInstanceStatus    : TIPSInterfaceStatus;
   { Instance Changed }
   FLastChange        : TDateTime;
   { Copy of Persistent needed Informations }
   FModuleInfo        : TIPSBasicModuleInformation;
   { Not avaiable via SOAP/PHP }
   FInstanceInterface : IIPSModule;
  public
   constructor Create; override;
   destructor Destroy; override;
   property InstanceInterface : IIPSModule read FInstanceInterface write FInstanceInterface;
  published
   property InstanceID        : TInstanceID read FInstanceID write FInstanceID;
   property ParentID          : TInstanceID read FParentID write FParentID;
   property NeedParent        : Boolean read FNeedParent write FNeedParent;
   property ChildrenIDs       : TInstanceIDs read FChildrenIDs write FChildrenIDs;
   property InstanceStatus    : TIPSInterfaceStatus read FInstanceStatus write FInstanceStatus;
   property LastChange        : TDateTime read FLastChange write FLastChange;
   property ModuleInfo        : TIPSBasicModuleInformation read FModuleInfo write FModuleInfo;
 end;
 TIPSInstances = Array of TIPSInstance;

//------------------------------------------------------------------------------
// Variable Handling
//------------------------------------------------------------------------------
 TIPSVarType = (vtBoolean, vtInteger, vtFloat, vtString, vtVariant, vtArray);
 TIPSVarTypes = set of TIPSVarType;

 TIPSVarIndexType = (itInt, itStr);

 TIPSVarIndex = class(TIPSRemotable)
  private
   FIndexType : TIPSVarIndexType;
   FIndexInt  : Integer;
   FIndexStr  : String;
  published
   property IndexType : TIPSVarIndexType read FIndexType write FIndexType;
   property IndexInt  : Integer read FIndexInt write FIndexInt;
   property IndexStr  : String read FIndexStr write FIndexStr;
 end;
 
 TIPSVarValue = class;
 TIPSVarValues = Array of TIPSVarValue;
 TIPSVarValue = class(TIPSRemotable)
  private
   FValueIndex   : TIPSVarIndex;
   FValueType    : TIPSVarType;
   FValueBoolean : Boolean;
   FValueInteger : Integer;
   FValueFloat   : Double;
   FValueString  : String;
   FValueVariant : Variant;
   FValueArray   : TIPSVarValues;  { Array Support }
   function PrepareAndCheckArrayEx(IndexType: TIPSVarIndexType; Name: String; Index: Integer): Integer;
   function PrepareAndCheckArrayStr(Name: String = ''): Integer;
   function PrepareAndCheckArrayInt(Index: Integer = -1): Integer;
  public
   constructor Create; override;
   destructor Destroy; override;

   procedure AddIndexedBool(Value: Boolean; Index: Integer = -1);
   procedure AddIndexedInt(Value: Integer; Index: Integer = -1);
   procedure AddIndexedFloat(Value: Double; Index: Integer = -1);
   procedure AddIndexedStr(Value: String; Index: Integer = -1);
   procedure AddIndexedVariant(Value: Variant; Index: Integer = -1);
   function  AddIndexedArray(Index: Integer = -1): TIPSVarValue;
   procedure AddNamedBool(Name: String; Value: Boolean);
   procedure AddNamedInt(Name: String; Value: Integer);
   procedure AddNamedFloat(Name: String; Value: Double);
   procedure AddNamedStr(Name: String; Value: String);
   procedure AddNamedVariant(Name: String; Value: Variant);
   function  AddNamedArray(Name: String): TIPSVarValue;
  published
   property ValueIndex   : TIPSVarIndex read FValueIndex write FValueIndex;
   property ValueType    : TIPSVarType read FValueType write FValueType;
   property ValueBoolean : Boolean read FValueBoolean write FValueBoolean;
   property ValueInteger : Integer read FValueInteger write FValueInteger;
   property ValueFloat   : Double read FValueFloat write FValueFloat;
   property ValueString  : String read FValueString write FValueString;
   property ValueVariant : Variant read FValueVariant write FValueVariant;
   property ValueArray   : TIPSVarValues read FValueArray write FValueArray;
 end;

 TIPSValueAssociation = class(TIPSRemotable)
  private
   FValue: Double;
   FName : String;
   FIcon : String;
  published
   property Value : Double read FValue write FValue;
   property Name  : String read FName write FName;
   property Icon  : String read FIcon write FIcon;
 end;
 TIPSValueAssociations = Array of TIPSValueAssociation;

 TIPSVarProfile = class(TIPSRemotable)
  private
   FProfileName  : String;
   FProfileType  : TIPSVarType;
   FPrefix       : String;
   FSuffix       : String;
   { Int/Float Only }
   FMinValue     : Double;
   FMaxValue     : Double;
   FStepSize     : Double;
   { Float only }
   FDigits       : Integer;
   { Bool/Int only [0=False,1=True] }
   FAssociations : TIPSValueAssociations;
   { Icon }
   FIcon         : String;
   { Flags }
   FReadOnly     : Boolean;
  public
   { Default destructor }
   destructor Destroy; override;
   { Special constructors }
   constructor CreateBooleanProfile(Name, Icon, TrueName, TrueIcon, FalseName, FalseIcon: String); reintroduce;
   constructor CreateIntegerProfile(Name, Icon, Prefix, Suffix: String; MinValue, MaxValue, StepSize: Integer); reintroduce;
   constructor CreateIntegerProfileEx(Name, Icon, Prefix, Suffix: String; Associations: TIPSValueAssociations); reintroduce;
   constructor CreateFloatProfile(Name, Icon, Prefix, Suffix: String; MinValue, MaxValue, StepSize: Double; Digits: Integer); reintroduce;
   constructor CreateFloatProfileEx(Name, Icon, Prefix, Suffix: String; Digits: Integer; Associations: TIPSValueAssociations); reintroduce;
   constructor CreateStringProfile(Name, Icon, Prefix, Suffix: String); reintroduce;
  published
   property ProfileName  : String read FProfileName write FProfileName;
   property ProfileType  : TIPSVarType read FProfileType write FProfileType;
   property Icon         : String read FIcon write FIcon;
   property Prefix       : String read FPrefix write FPrefix;
   property Suffix       : String read FSuffix write FSuffix;
   property MinValue     : Double read FMinValue write FMinValue;
   property MaxValue     : Double read FMaxValue write FMaxValue;
   property StepSize     : Double read FStepSize write FStepSize;
   property Digits       : Integer read FDigits write FDigits;
   property Associations : TIPSValueAssociations read FAssociations write FAssociations;
   property IsReadOnly   : Boolean read FReadOnly write FReadOnly;
 end;
 TIPSVarProfiles = Array of TIPSVarProfile;

 TIPSVariable = class(TIPSRemotable)
  private
   { Identify the Variable }
   FVariableID               : TVariableID;
   { Variable Configuration }
   FVariableProfile          : String;
   FVariableCustomProfile    : String;
   FVariableCustomAction     : TScriptID;
   FVariableUpdated          : TDateTime;
   FVariableChanged          : TDateTime;
   { Variable Value + Type }
   FVariableValue            : TIPSVarValue;
   { Variable Flag }
   FVariableIsBinary         : Boolean;
   { Variable Notify Settings }
   FVariableNotifyDebounce   : Integer; //Seconds, 0 = Always
   FVariableNotifyThreshold  : Integer; //% Change needed for Event
  public
   constructor Create; override;
   destructor Destroy; override;
  published
   { Identify the Variable }
   property VariableID               : TVariableID read FVariableID write FVariableID;
   { Variable Configuration }

   {
     Version 3.x -> VariableProfile rauswerfen, da es eigentlich hier nicht hingehrt.
     Es muss in der StatusVariable von der Instanz gespeichert sein und von dort aus
     aufgelst werden. Es kann keine direkte verknpfung stattfinden, da eine Variable
     von mehreren Instanzen eine Status Variable sein kann
   }
   
   property VariableProfile          : String read FVariableProfile write FVariableProfile;
   property VariableCustomProfile    : String read FVariableCustomProfile write FVariableCustomProfile;
   property VariableCustomAction     : TScriptID read FVariableCustomAction write FVariableCustomAction;
   { Variable Timestamps }
   property VariableUpdated          : TDateTime read FVariableUpdated write FVariableUpdated;
   property VariableChanged          : TDateTime read FVariableChanged write FVariableChanged;
   { Variable Value + Type }
   property VariableValue            : TIPSVarValue read FVariableValue write FVariableValue;
   { Variable Flag }
   property VariableIsBinary         : Boolean read FVariableIsBinary write FVariableIsBinary;

 end;
 TIPSVariables = Array of TIPSVariable;

//------------------------------------------------------------------------------
// Event Handling
//------------------------------------------------------------------------------
 TIPSEventType = (etTrigger, etCyclic);

 TIPSTriggerType = (evtOnUpdate, evtOnChange, evtOnLimitExceed, evtOnLimitDrop, evtOnValue);
 TIPSCyclicDateType = (cdtNone, cdtOnce, cdtDay, cdtWeek, cdtMonth, cdtYear);
 TIPSCyclicTimeType = (cttOnce, cttSecond, cttMinute, cttHour, cttSpecial);

 TIPSEvent=class(TIPSRemotable)
  private
   FEventID: TIPSID;
   FEventType: TIPSEventType;
   FEventActive: Boolean;                 //Active/Inactive
   FEventLimit: Integer;                  //0 = Unlimited, >0 Times left
   //---
   FTriggerType: TIPSTriggerType;
   FTriggerVariableID: TVariableID;
   FTriggerValue: Variant;
   FTriggerSubsequentExecution: Boolean;
   //---
   FCyclicDateType: TIPSCyclicDateType;
   FCyclicDateFrom: TDateTime;            //Timer Valid from, 0 = NOW
   FCyclicDateTo: TDateTime;              //Timer Valid to, 0 = ENDLESS
   FCyclicDateValue: Integer;             //Each x {Day, Week, Month, Year}
   FCyclicDateDay: Integer;               //Each Mon(2^0), Tue(2^1), Wed(2^2)...
   FCyclicDateDayValue: Integer;          //Each x "DateDay" Day
   //---
   FCyclicTimeType: TIPSCyclicTimeType;
   FCyclicTimeFrom: TDateTime;            //Timer Valid from, 0 = NOW
   FCyclicTimeTo: TDateTime;              //Timer Valid to, 0 = ENDLESS
   FCyclicTimeValue: Integer;             //Each x {Second,Minute...}
   //---
   FEventScript: String;
   FLastRun: TDateTime;                   //0 = NEVER
   FNextRun: TDateTime;
  published
   //Properties
   property EventID       : TIPSID read FEventID write FEventID;
   property EventType     : TIPSEventType read FEventType write FEventType;
   property EventActive   : Boolean read FEventActive write FEventActive;
   property EventLimit    : Integer read FEventLimit write FEventLimit;

   property TriggerType : TIPSTriggerType read FTriggerType write FTriggerType;
   property TriggerVariableID : TVariableID read FTriggerVariableID write FTriggerVariableID;
   property TriggerValue    : Variant read FTriggerValue write FTriggerValue;
   property TriggerSubsequentExecution: Boolean read FTriggerSubsequentExecution write FTriggerSubsequentExecution;

   property CyclicDateType  : TIPSCyclicDateType read FCyclicDateType write FCyclicDateType;
   property CyclicDateFrom  : TDateTime read FCyclicDateFrom write FCyclicDateFrom;
   property CyclicDateTo    : TDateTime read FCyclicDateTo write FCyclicDateTo;
   property CyclicDateValue : Integer read FCyclicDateValue write FCyclicDateValue;
   property CyclicDateDay      : Integer read FCyclicDateDay write FCyclicDateDay;
   property CyclicDateDayValue : Integer read FCyclicDateDayValue write FCyclicDateDayValue;

   property CyclicTimeType  : TIPSCyclicTimeType read FCyclicTimeType write FCyclicTimeType;
   property CyclicTimeFrom  : TDateTime read FCyclicTimeFrom write FCyclicTimeFrom;
   property CyclicTimeTo    : TDateTime read FCyclicTimeTo write FCyclicTimeTo;
   property CyclicTimeValue : Integer read FCyclicTimeValue write FCyclicTimeValue;

   property EventScript     : String read FEventScript write FEventScript;
   property LastRun : TDateTime read FLastRun write FLastRun;
   property NextRun : TDateTime read FNextRun write FNextRun;
 end;
 TIPSEvents = Array of TIPSEvent;

//------------------------------------------------------------------------------
// Script Handling
//------------------------------------------------------------------------------
 TIPSScriptType = (stPHPScript, stMacroScript, stBrickScript);

 TIPSScript = class(TIPSRemotable)
  private
   { Identify the Script }
   FScriptID    : TScriptID;
   { Script Configuration }
   FScriptType  : TIPSScriptType;
   FScriptFile  : String;
   FIsBroken    : Boolean;    //Will be set to TRUE if an Error occours and FALSE by the next successful run
   FLastExecute : TDateTime;
  published
   { Identify the Script }
   property ScriptID    : TScriptID read FScriptID write FScriptID;
   { Script Configuration }
   property ScriptType  : TIPSScriptType read FScriptType write FScriptType;
   property ScriptFile  : String read FScriptFile write FScriptFile;
   property IsBroken    : Boolean read FIsBroken write FIsBroken;              //Will be set to TRUE if an Error occours and FALSE by the next successful run
   property LastExecute : TDateTime read FLastExecute write FLastExecute;
 end;
 TIPSScripts = Array of TIPSScript;

//------------------------------------------------------------------------------
// Script Function/Data Handling
//------------------------------------------------------------------------------
 TIPSPHPVariable = class(TIPSRemotable)
  private
   FVariableName : String;
   FVariableValue: TIPSVarValue;
  public
   constructor Create; override;
   constructor CreateNamed(Name: String); reintroduce;
   constructor CreateBool(Name: String; Value: Boolean); reintroduce;
   constructor CreateInt(Name: String; Value: Integer); reintroduce;
   constructor CreateFloat(Name: String; Value: Double); reintroduce;
   constructor CreateStr(Name: String; Value: String); reintroduce;  
   destructor Destroy; override;   
  published
   property VariableName : String read FVariableName write FVariableName;
   property VariableValue: TIPSVarValue read FVariableValue write FVariableValue;
 end;
 TIPSPHPVariables = Array of TIPSPHPVariable; 

 TIPSPHPParameter = TIPSVarValue;
 TIPSPHPParameters = TIPSVarValues;

 TIPSPHPExecute = procedure(FunctionName    : String;
                            Parameters      : TIPSPHPParameters;
                            var ReturnValue : TIPSPHPParameter) of Object;

 TIPSPHPResultType = TIPSVarType;
 TIPSPHPParameterInfo = class(TIPSRemotable)
  private
   FDescription : String;
   FType        : TIPSPHPResultType;
   FEnumeration : TStringArray;
  published
   property Description : String read FDescription write FDescription;
   property Type_ : TIPSPHPResultType read FType write FType;
   property Enumeration : TStringArray read FEnumeration write FEnumeration;
 end;
 TIPSPHPParameterInfos = Array of TIPSPHPParameterInfo;

 TIPSPHPInstance = class(TIPSRemotable)
  private
   FInstanceID      : TInstanceID;
   FSingleton       : Boolean;
   FFunctionHandler : TIPSPHPExecute;
  public 
   property FunctionHandler : TIPSPHPExecute read FFunctionHandler write FFunctionHandler;
  published
   property InstanceID : TInstanceID read FInstanceID write FInstanceID;
   property Singleton  : Boolean read FSingleton write FSingleton;
 end;
 TIPSPHPInstances = Array of TIPSPHPInstance;

 TIPSPHPFunction = class(TIPSRemotable)
  protected
   FFunctionName : String;
   FParameters   : TIPSPHPParameterInfos;
   FResult       : TIPSPHPParameterInfo;
   FPHPInstances : TIPSPHPInstances;
   function GetParameters : TIPSPHPParameterInfos; virtual;
   procedure SetParameters(Value: TIPSPHPParameterInfos); virtual;
   function GetInstances: TIPSPHPInstances; virtual;
   procedure SetInstances(Value: TIPSPHPInstances); virtual;
  public
   constructor Create; override;
   destructor Destroy; override;   
  published
   property FunctionName : String read FFunctionName write FFunctionName;
   property Parameters   : TIPSPHPParameterInfos read GetParameters write SetParameters;
   property Result       : TIPSPHPParameterInfo read FResult write FResult;
   property Instances    : TIPSPHPInstances read GetInstances write SetInstances;
 end;
 TIPSPHPFunctions = Array of TIPSPHPFunction;
 
//------------------------------------------------------------------------------
// Script Result Handling
//------------------------------------------------------------------------------
 TIPSExecuteInfo = class;
 
 TReqMethod = (rmGET, rmPOST);
 TOnNewContent = procedure(Content: String; Data: TIPSExecuteInfo) of object; 

 TIPSExecuteInfo = class(TIPSRemotable)
  private
   //Input
   FRequestMethod   : TReqMethod;
   FQueryString     : String;
   FServerVariables : TIPSPHPVariables;
   FCookies         : String;
   //Post
   FPostData        : TMemoryStream;
   FContentType     : String;
   FContentLength   : Integer;
   //Output
   FScriptResult    : String;
   FScriptHeaders   : TStringArray;
   //Event
   FOnNewContent    : TOnNewContent;
   //Data
   FData            : Pointer;
  public
   constructor Create; override;
   destructor Destroy; override;
  public
   property OnNewContent    : TOnNewContent read FOnNewContent write FOnNewContent;
   property Data            : Pointer read FData write FData;
   //Input
   property RequestMethod   : TReqMethod read FRequestMethod write FRequestMethod;
   property QueryString     : String read FQueryString write FQueryString;
   property Cookies         : String read FCookies write FCookies;
   //Post
   property PostData        : TMemoryStream read FPostData write FPostData;
   property ContentType     : String read FContentType write FContentType;
   property ContentLength   : Integer read FContentLength write FContentLength;
  published
   //Input
   property ServerVariables : TIPSPHPVariables read FServerVariables write FServerVariables;
   //Output
   property ScriptResult    : String read FScriptResult write FScriptResult;
   property ScriptHeaders   : TStringArray read FScriptHeaders write FScriptHeaders;
 end;

//------------------------------------------------------------------------------
// Media Handling
//------------------------------------------------------------------------------
 TIPSMediaType = (mtForm, mtImage, mtSound, mtStream);

 TIPSMedia = class(TIPSRemotable)
  private
   { Identify the Media }
   FMediaID     : TMediaID;
   { Media Con figuration }
   FMediaType   : TIPSMediaType;
   FMediaFile   : String;
   FMediaCRC    : String;
   FMediaSize   : Integer;
   FIsAvailable : Boolean;    //Will be set to FALSE if Media is unavailable
   FSendEvent   : Boolean;
   FLastUpdate  : TDateTime;
  published
   { Identify the Media }
   property MediaID     : TMediaID read FMediaID write FMediaID;
   { Media Configuration }
   property MediaType   : TIPSMediaType read FMediaType write FMediaType;
   property MediaFile   : String read FMediaFile write FMediaFile;
   property MediaCRC    : String read FMediaCRC write FMediaCRC;
   property MediaSize   : Integer read FMediaSize write FMediaSize;
   property IsAvailable : Boolean read FIsAvailable write FIsAvailable;
   property SendEvent   : Boolean read FSendEvent write FSendEvent;
   property LastUpdate  : TDateTime read FLastUpdate write FLastUpdate;
 end;
 TIPSMedias = Array of TIPSMedia;

//------------------------------------------------------------------------------
// Link Handling
//------------------------------------------------------------------------------
 TIPSLink = class(TIPSRemotable)
  private
   { Identify the Link }
   FLinkID      : TLinkID;
   { Link Configuration }
   FLinkChildID : TIPSID;
  published
   { Identify the Link }
   property LinkID      : TLinkID read FLinkID write FLinkID;
   { Media Configuration }
   property LinkChildID : TIPSID read FLinkChildID write FLinkChildID;
 end;
 TIPSLinks = Array of TIPSLink;

//------------------------------------------------------------------------------
// Timer Handling
//------------------------------------------------------------------------------
 TIPSTimer = class(TIPSRemotable)
  private
   FTimerID: TTimerID;
   FInstanceID: TInstanceID;
   FName: String;
   FInterval: Integer;
   { Internal Data }
   FEvent: TTimerEvent;
   { Dynamic Data }
   FLastRun: TDateTime;
   FNextRun: TDateTime;
   FRunning: Boolean;
  public
   property Event      : TTimerEvent read FEvent write FEvent;
  published
   { Identify the Timer }
   property TimerID    : TTimerID read FTimerID write FTimerID;
   property InstanceID : TInstanceID read FInstanceID write FInstanceID;
   property Name       : String read FName write FName;
   { Timer Configuration }
   property Interval   : Integer read FInterval write FInterval;
   property LastRun    : TDateTime read FLastRun write FLastRun;
   property NextRun    : TDateTime read FNextRun write FNextRun;
   property Running    : Boolean read FRunning write FRunning;
 end;
 TIPSTimers = Array of TIPSTimer;

//------------------------------------------------------------------------------
// Thread Handling
//------------------------------------------------------------------------------
 TIPSScriptThreadInfo = class(TIPSRemotable)
  private
   FThreadID     : THandle;
   FExecuteCount : Cardinal;
   FStartTime    : TDateTime;
   FSender       : String;
   FFilePath     : String;
   FScriptID     : TScriptID;
  published
   property ThreadID     : THandle read FThreadID write FThreadID;
   property ExecuteCount : Cardinal read FExecuteCount write FExecuteCount;
   property StartTime    : TDateTime read FStartTime write FStartTime;
   property Sender       : String read FSender write FSender;
   property FilePath     : String read FFilePath write FFilePath;
   property ScriptID     : TScriptID read FScriptID write FScriptID;
 end;
 TIPSScriptThreadInfos = Array of TIPSScriptThreadInfo;

//------------------------------------------------------------------------------
// ModuleObejct Handling
//------------------------------------------------------------------------------
 TIPSStatusVariable = class;

 TIPSActionRequestHandler = procedure(StatusVariable: TIPSStatusVariable; Value: Variant) of object;

 TIPSStatusVariable = class(TIPSRemotable)
  private
   FVariableIdent    : String;
   FVariableName     : String; //This is the Default Name
   FVariableType     : TIPSVarType;
   FVariableProfile  : String;
   FVariablePosition : Integer;
   //Can be assigned by User
   FVariableID      : TVariableID;
   FActionHandler   : TIPSActionRequestHandler;
   FHasAction       : Boolean;
   FUseAction       : Boolean;
   procedure SetActionHandler(Value: TIPSActionRequestHandler);
  public
   property ActionHandler : TIPSActionRequestHandler read FActionHandler write SetActionHandler;
  published
   property VariableIdent : String read FVariableIdent write FVariableIdent;
   property VariableName : String read FVariableName write FVariableName;
   property VariableType : TIPSVarType read FVariableType write FVariableType;
   property VariableProfile: String read FVariableProfile write FVariableProfile;
   property VariablePosition: Integer read FVariablePosition write FVariablePosition;
   property VariableID: TVariableID read FVariableID write FVariableID;
   property VariableHasAction: Boolean read FHasAction write FHasAction;
   property VariableUseAction: Boolean read FUseAction write FUseAction;
 end;
 TIPSStatusVariables = Array of TIPSStatusVariable;

//------------------------------------------------------------------------------
// DebugData Handling
//------------------------------------------------------------------------------
 TIPSDebugMessage = class(TIPSRemotable)
  private
   FTimeStamp: TDateTime;
   FInstanceID: TInstanceID;
   FMessage: String;
   FData: String;
  public
   constructor Create(InstanceID: TInstanceID; Message, Data: String); reintroduce; overload;
   constructor Create(Msg: TIPSDebugMessage); reintroduce; overload;
  published
   property TimeStamp: TDateTime read FTimeStamp write FTimeStamp;
   property InstanceID: TInstanceID read FInstanceID write FInstanceID;
   property Message: String read FMessage write FMessage;
   property Data: String read FData write FData;
 end;
 TIPSDebugMessages = Array of TIPSDebugMessage;

//------------------------------------------------------------------------------
// SoapServer Handling
//------------------------------------------------------------------------------
 TIPSFilterType = (ftIncludeFilter, ftExcludeFilter);

 TIPSMessageFilter = class(TIPSRemotable)
  private
   FSenderID: Integer;
   FMessage: Integer;
  public
   constructor Create(SenderID, Message: Integer); reintroduce;
  published
   property SenderID: Integer read FSenderID write FSenderID;
   property Message: Integer read FMessage write FMessage;
 end;
 TIPSMessageFilters = Array of TIPSMessageFilter;
 
//------------------------------------------------------------------------------
// Interfaces
//------------------------------------------------------------------------------
// --- ModuleRegister Interface
 IIPSModuleRegistry=interface(IInterface)
  ['{66BFF782-3CCA-4FD9-9B12-906C12852618}']
   { Register a Module Class }
   procedure RegisterModule(ModuleClass: TIPSModuleClass; InterfaceInfo: PTypeInfo; Prefix: String = ''; Singleton: Boolean = False); stdcall;
  end;

 IIPSInterface = interface(IInterface)
  ['{C7CA6A65-CE8E-46FD-9976-D1FF9C9C0F91}']
  { Reference Counting }
  function ReferenceCount(): Integer;
 end;

// --- MessageSink Interface
 IIPSMessageSink=interface(IIPSInterface)
   ['{4AB6071E-F8D1-4968-A432-17751D071BF8}']
   procedure MessageSink(var Msg: TIPSMessage);
  end;

// --- Module Interface
 IIPSModule = interface(IIPSMessageSink)
  ['{551860D7-8E44-430B-8C44-CBB5AA708470}']
  { Status Variables }
  function StatusVariableExists(VariableIdent: String): Boolean; stdcall;
  function GetStatusVariable(VariableIdent: String): TIPSStatusVariable; stdcall;
  function GetStatusVariables(): TIPSStatusVariables; stdcall;
  function GetStatusVariableIdents(): TStringArray; stdcall;
  procedure SetStatusVariableUseAction(VariableIdent: String; UseAction: Boolean); stdcall;
  function GetStatusVariableID(VariableIdent: String): TVariableID; stdcall;
  procedure SetStatusVariableID(VariableIdent: String; VariableID: TVariableID); stdcall;
  procedure RequestAction(VariableIdent: String; Value: Variant); stdcall;
  { Searching }
  function SupportsSearching: Boolean; stdcall;
  procedure DoSearching(Enable: Boolean); stdcall;
  procedure SelectDevice(SearchID: Integer); stdcall;
  function IsSearching: Boolean; stdcall;
  { Will load Settings }
  procedure LoadSettings(); stdcall;
  { Will save Settings }
  procedure SaveSettings(); stdcall;
  { Will reset/Apply current Settings }
  function HasChanges: Boolean; stdcall;
  procedure ResetChanges(); stdcall;
  procedure ApplyChanges(); stdcall;
  { InstanceID }
  function GetInstanceID(): TInstanceID; stdcall;
 end;

// --- Kernel Interface
 IIPSKernel=interface(IIPSMessageSink)
  ['{BCB79E07-1AA9-42B1-B154-EDC5AFB50D37}']
  (*
    Info: Will load all needed Interfaces
    Messages: IPS_KERNELMESSAGE => KR_INIT(Const) | IPS_KERNELMESSAGE => KR_READY(Const)
    RunningTime: -
  *)
  procedure InitKernel; stdcall;

  (*
    Info: Will unload all Interfaces
    Messages: IPS_KERNELMESSAGE => KR_UNINIT(Const), IPS_KERNELMESSAGE => KR_SHUTDOWN(Const)
    RunningTime: -
  *)
  procedure UninitKernel; stdcall;

  (*
    Info: Returns the current Kernel Version
    Messages: -
    RunningTime: O(1)
  *)
  function GetKernelVersion: Word; stdcall;

  (*
    Info: Displays some Message to KernelLog
    Type: HiByte - MajorV, LoByte - MinorV
    Messages: IPS_LOGMESSAGE => Type(Int), Sender(Str), Message(Str)
    RunningTime: O(1)
  *)
  procedure LogMessage(LType: Integer; LSenderID: TIPSID; LSender: String; LMessage: String); stdcall;

  (*
    Info: Send Message to all Instances
    Messages: -
    RunningTime: At least O(n) | n = Instance Count
  *)
  procedure SendMessage(iMsg: TIPSMessage); stdcall;

  (*
    Info: Send Message to the internal Message Queue
    Messages: -
    RunningTime: O(1)
  *)
  procedure PostMessage(SenderID: TIPSID; Message: Integer; Data: Array of Const); stdcall;
  
  (*
    Info: Returns the current KernelDir
    Messages: -
    RunningTime: O(1)
  *)
  function GetKernelDir: String; stdcall;

  (*
    Info: Returns the current Kernel Runlevel
    Messages: -
    RunningTime: O(1)
  *)
  function GetKernelRunlevel: Integer; stdcall;

  (*
    Info: Returns the last Kernel Ticktime (updated every Seconds from the Main Thread)
    Messages: -
    RunningTime: O(1)
  *)
  function GetKernelTicktime: TDateTime; stdcall;

  (*
    Info: Returns Kernel SubModules
    Messages: -
    RunningTime: O(1)
  *)
  function GetModuleLoaderEx: IIPSModuleLoaderEx; stdcall;
  function GetInstanceManagerEx: IIPSInstanceManagerEx; stdcall;
  function GetVariableManagerEx: IIPSVariableManagerEx; stdcall;
  function GetScriptEngineEx: IIPSScriptEngineEx; stdcall;
  function GetTimerPoolEx: IIPSTimerPoolEx; stdcall;
  function GetFilePoolEx: IIPSFilePoolEx; stdcall;
  (*
    Info: Property declarations for Get* Functions
    Messages: -
    RunningTime: Same as Get* Functions
  *)
  property ModuleLoaderEx: IIPSModuleLoaderEx read GetModuleLoaderEx;
  property InstanceManagerEx: IIPSInstanceManagerEx read GetInstanceManagerEx;
  property VariableManagerEx: IIPSVariableManagerEx read GetVariableManagerEx;
  property ScriptEngineEx: IIPSScriptEngineEx read GetScriptEngineEx;
  property TimerPoolEx: IIPSTimerPoolEx read GetTimerPoolEx;
  property FilePoolEx: IIPSFilePoolEx read GetFilePoolEx;

  (*
    Info: Returns Kernel SubModules
    Messages: -
    RunningTime: O(1)
  *)
  function GetSimpleKernel: IIPSSimpleKernel; stdcall;
  function GetModuleLoader: IIPSModuleLoader; stdcall;
  function GetObjectManager: IIPSObjectManager; stdcall;
  function GetCategoryManager: IIPSCategoryManager; stdcall;
  function GetInstanceManager: IIPSInstanceManager; stdcall;
  function GetVariableManager: IIPSVariableManager; stdcall;
  function GetScriptManager: IIPSScriptManager; stdcall;
  function GetEventManager: IIPSEventManager; stdcall;
  function GetMediaManager: IIPSMediaManager; stdcall;
  function GetLinkManager: IIPSLinkManager; stdcall;
  function GetScriptEngine: IIPSScriptEngine; stdcall;
  function GetDataHandler: IIPSDataHandler; stdcall;
  function GetTimerPool: IIPSTimerPool; stdcall;
  function GetFilePool: IIPSFilePool; stdcall;
  function GetSettings: IIPSSettings; stdcall;
  function GetDebugServer: IIPSDebugServer; stdcall;
  function GetSOAPServer: IIPSSOAPServer; stdcall;
  function GetLicensePool: IIPSLicensePool; stdcall;

  (*
    Info: Property declarations for Get* Functions
    Messages: -
    RunningTime: Same as Get* Functions
  *)
  property SimpleKernel: IIPSSimpleKernel read GetSimpleKernel;
  property ModuleLoader: IIPSModuleLoader read GetModuleLoader;
  property ObjectManager: IIPSObjectManager read GetObjectManager;
  property CategoryManager: IIPSCategoryManager read GetCategoryManager;
  property InstanceManager: IIPSInstanceManager read GetInstanceManager;
  property VariableManager: IIPSVariableManager read GetVariableManager;
  property ScriptManager: IIPSScriptManager read GetScriptManager;
  property EventManager: IIPSEventManager read GetEventManager;
  property MediaManager: IIPSMediaManager read GetMediaManager;
  property LinkManager: IIPSLinkManager read GetLinkManager;
  property ScriptEngine: IIPSScriptEngine read GetScriptEngine;
  property TimerPool: IIPSTimerPool read GetTimerPool;
  property FilePool: IIPSFilePool read GetFilePool;
  property DataHandler: IIPSDataHandler read GetDataHandler;
  property Settings: IIPSSettings read GetSettings;
  property DebugServer: IIPSDebugServer read GetDebugServer;
  property SOAPServer: IIPSSOAPServer read GetSOAPServer;
  property LicensePool: IIPSLicensePool read GetLicensePool;

  (*
  *)
  function IsRemoteCall(): Boolean;
 end;

// --- SimpleKernel Interface
 IIPSSimpleKernel=interface(IIPSMessageSink)
  ['{CE014520-8DBA-4225-9C34-1F94100D1D00}']
  (*
    Info: Returns the current Kernel Version
    Messages: -
    RunningTime: O(1)
  *)
  function GetKernelVersion: Word; stdcall;

  (*
    Info: Displays some Message to KernelLog
    Type: HiByte - MajorV, LoByte - MinorV
    Messages: IPS_LOGMESSAGE => Type(Int), Sender(Str), Message(Str)
    RunningTime: O(1)
  *)
  procedure LogMessage(LType: Integer; LSenderID: TIPSID; LSender: String; LMessage: String); stdcall;

  (*
    Info: Returns the current KernelDir
    Messages: -
    RunningTime: O(1)
  *)
  function GetKernelDir: String; stdcall;

  (*
    Info: Returns the current Kernel Runlevel
    Messages: -
    RunningTime: O(1)
  *)
  function GetKernelRunlevel: Integer; stdcall;

  (*
    Info: Returns the last Kernel Ticktime (updated every Seconds from the Main Thread)
    Messages: -
    RunningTime: O(1)
  *)  
  function GetKernelTicktime: TDateTime; stdcall;
 end;

// --- LicensePool Interfaces
 IIPSLicensePool=interface(IIPSMessageSink)
  ['{1674C142-C6C0-485B-897B-BC1423C87E8D}']

  (*
  *)
  function GetLicensee: String; stdcall;

  (*
  *)
  function GetLiveUpdateVersion: String; stdcall;

  (*
  *)
  function GetLimitSystems: Integer; stdcall;

  (*
  *)
  function GetLimitInstances: Integer; stdcall;

  (*
  *)
  function GetLimitLogging: Integer; stdcall;

  (*
  *)
  function GetLimitUsers: Integer; stdcall;
 end;

// --- ModuleLoader Interfaces
 IIPSModuleLoader=interface(IIPSMessageSink)
  ['{9A0F6FD4-1719-451D-900D-C13250995753}']
  (*
    Info: Loads not yet loaded Libraries
    Messages: -
    RunningTime: O(n*m) | n = DLL's in ModuleFolder, m = LoadLibrary() RunningTime
  *)
  procedure LoadLibraries; stdcall;

  (*
    Info: Unloads all loaded Libraries
    Messages: -
    RunningTime: O(n*m) | n = DLL's in ModuleFolder, m = LoadLibrary() RunningTime
  *)
  procedure UnloadLibraries; stdcall;

  (*
    Info: Load Library
    Messages: IPS_MODULEMESSAGE => ML_LOAD(Const), GUID(Str)
    RunningTime: O(n+m) | n = CurrentModuleCount, m = IIPSInstanceControl.ConnectModuleInstances() RunningTime
  *)
  procedure LoadLibrary(Filename: String); stdcall;

  (*
    Info: Unloads Library
    Messages: IPS_MODULEMESSAGE => ML_UNLOAD(Const), GUID(Str)
    RunningTime: O(n+m) | n = CurrentModuleCount, m = IIPSInstanceControl.DisconnectModuleInstances() RunningTime
  *)
  procedure UnloadLibrary(LibraryID: TStrGUID); stdcall;

  (*
    Info: Return if Library for LibraryID exits
    Messages: -
    RunningTime: O(1)
  *)
  function LibraryExists(LibraryID: TStrGUID): Boolean; stdcall;

  (*
    Info: Return Pointer to Library for LibraryID | Do not Modify Record!
    Messages: -
    RunningTime: O(1)
  *)
  function GetLibrary(LibraryID: TStrGUID): TIPSLibraryInformation; stdcall;

  (*
    Info: Return GUIDs to all Libraries
    Messages: -
    RunningTime: O(n) | n = CurrentLibraryCount
  *)
  function GetLibraries(LibraryIDs: TStrGUIDs): TIPSLibraryInformations; stdcall;

  (*
    Info: Return GUIDs to all Libraries
    Messages: -
    RunningTime: O(n) | n = CurrentLibraryCount
  *)
  function GetLibraryList(): TStrGUIDs; stdcall;

  (*
    Info: Return GUIDs to all Modules from Library
    Messages: -
    RunningTime: O(n+m) | n = CurrentLibraryCount, m = CurrentModuleCount
  *)
  function GetLibraryModules(LibraryID: TStrGUID): TStrGUIDs; stdcall;

  (*
    Info: Return if Module for ModuleID exits
    Messages: -
    RunningTime: O(1)
  *)
  function ModuleExists(ModuleID: TStrGUID): Boolean; stdcall;

  (*
   Info: Collect all Module Information

  *)
  function GetModule(ModuleID: TStrGUID): TIPSModuleInformation; stdcall;
  
  (*
    Info: Return GUIDs to all Modules
    Messages: -
    RunningTime: O(n) | n = CurrentModuleCount
  *)
  function GetModules(ModuleIDs: TStrGUIDs): TIPSModuleInformations; stdcall;

  (*
    Info: Return GUIDs to all Modules
    Messages: -
    RunningTime: O(n) | n = CurrentModuleCount
  *)
  function GetModuleList(): TStrGUIDs; stdcall;

  (*
   Info: Checks if both Modules can be connected
   Messages: -
   RunningTime: O(1)
  *)
  function IsModuleCompatible(ModuleID, ParentModuleID: TStrGUID): Boolean; stdcall;

  (*
   Info: Searches all compatible Modules can be connected with Module
   Messages: -
   RunningTime: O(1)
  *)
  function GetCompatibleModules(ModuleID: TStrGUID): TStrGUIDs; stdcall;
  
 end;

 IIPSModuleLoaderEx = interface(IIPSMessageSink)
  ['{18D0B53E-865F-48EA-A227-39327457C318}']
  (*
    Info: Returns ModuleClass. Do not create Instaces of it. Use InstanceControl.CreateInstance
    Messages: -
    RunningTime: O(1)
  *)
  function GetModuleClass(ModuleID: TStrGUID): TIPSModuleClass; stdcall;
  (*
    Info: Returns ModulePrefix.
    Messages: -
    RunningTime: O(1)
  *)
  function GetModulePrefix(ModuleID: TStrGUID): String; stdcall;
  (*
    Info: Returns if Module is singleton.
    Messages: -
    RunningTime: O(1)
  *)
  function GetModuleSingleton(ModuleID: TStrGUID): Boolean; stdcall;
  (*
    Info: Returns InterfaceInfo.
    Messages: -
    RunningTime: O(n)
  *)
  function GetInterfaceInfo(InterfaceID: TGUID): PTypeInfo; stdcall;  
 end;
 
// --- DataHandler Interface
 IIPSDataHandler=interface(IIPSMessageSink)
  ['{3FF06499-9CF9-42AE-A2F0-E9C9C3ED60DA}']
  { Connects the Instance with some Parent Instance, 0 = Connect to None }
  procedure ConnectInstance(InstanceID, ParentID: TInstanceID); stdcall;
  { Disconnects the Instance from all other ones }
  procedure DisconnectInstance(InstanceID: TInstanceID); stdcall;
  { Has Parent? }
  function HasInstanceParent(InstanceID: TInstanceID): Boolean; stdcall;
  { Return ParentID for InstanceID }
  function GetInstanceParentID(InstanceID: TInstanceID): TInstanceID; stdcall;
  { Return all ParentIDs for InstanceID }
  function GetInstanceParentIDs(InstanceID: TInstanceID): TInstanceIDs; stdcall;
  { Has Children? }
  function HasInstanceChildren(InstanceID: TInstanceID): Boolean; stdcall;
  { Only applicable for Splitters and Devices }
  function GetInstanceChildrenIDs(InstanceID: TInstanceID): TInstanceIDs; stdcall;
 end;

// --- ObjectManager Interface
 IIPSObjectManager=interface(IIPSMessageSink)
  ['{085EA06E-0143-4EFB-B8C5-794542F58D04}']
  { Registers Object and returns it's ID }
  function RegisterNewObject(ObjectType: TIPSObjectType; Persistent: Boolean = True): TIPSID; stdcall;
  { Registers Object }
  procedure RegisterObject(ObjectType: TIPSObjectType); stdcall;
  { Registers Object and using given ID }
  procedure RegisterObjectEx(ObjectType: TIPSObjectType; ID: TIPSID); stdcall;
  { Unregisters the given Object and updates all Children with a valid ParentID }
  procedure UnregisterObject(ID: TIPSID); stdcall;
  { Set the Objects Name/Parent }
  procedure SetParent(ID: TIPSID; ParentID: TIPSID); stdcall;
  procedure SetName(ID: TIPSID; Name: String); stdcall;
  procedure SetInfo(ID: TIPSID; Info: String); stdcall;
  procedure SetIcon(ID: TIPSID; Icon: String); stdcall;
  procedure SetSummary(ID: TIPSID; Summary: String); stdcall;
  procedure SetPosition(ID: TIPSID; Position: Integer); stdcall;
  procedure SetReadOnly(ID: TIPSID; ReadOnly: Boolean); stdcall;
  procedure SetHidden(ID: TIPSID; Hidden: Boolean); stdcall;
  { Getter Functions }
  function ObjectExists(ID: TIPSID): Boolean; stdcall;
  procedure CheckObject(ID: TIPSID); stdcall;
  function GetObject(ID: TIPSID): TIPSObject; stdcall;
  function GetObjects(IDs: TIPSIDs): TIPSObjects; stdcall;
  function GetObjectList(): TIPSIDs; stdcall;
  function GetObjectIDByName(Name: String; ParentID: TIPSID): TIPSID; stdcall;
  function GetObjectIDByNameEx(Name: String; ParentID: TIPSID; ObjectType: TIPSObjectType): TIPSID; stdcall;
  function HasChildren(ID: TIPSID): Boolean; stdcall;
  function IsChild(ID: TIPSID; Parent: TIPSID; Recursive: Boolean = True): Boolean; stdcall;
  function GetChildrenIDs(ID: TIPSID): TIPSIDs; stdcall;
  function IsPersistent(ID: TIPSID): Boolean; stdcall;
  function GetName(ID: TIPSID): String; stdcall;
  function GetParent(ID: TIPSID): TIPSID; stdcall;
  function GetLocation(ID: TIPSID): String; stdcall;
  function GetLocations(IDs: TIPSIDs): TStringArray; stdcall;
 end;

//-- CategoryManager Interface
 IIPSCategoryManager=interface(IIPSMessageSink)
  ['{EF1146D4-0DFC-4293-928D-C55EEEF7AD00}']
  function CreateCategory(): TCategoryID; stdcall;
  function CreateCategoryEx(ID: TCategoryID): TCategoryID; stdcall;
  procedure DeleteCategory(ID: TCategoryID); stdcall;
  { Set the Objects Name/Parent }
//FIXME: Determine if needed  procedure SetCategoryBackground(ID: TCategoryID; MediaID: TMediaID); stdcall;
  { Getter Functions }
  function CategoryExists(ID: TCategoryID): Boolean; stdcall;
  procedure CheckCategory(ID: TCategoryID); stdcall;
  function GetCategoryList(): TCategoryIDs; stdcall;
  function GetCategoryIDByName(Name: String; ParentID: TIPSID): TCategoryID; stdcall;
 end;

//-- InstanceManager Interface
 IIPSInstanceManager=interface(IIPSMessageSink)
  ['{0D8CEA36-DFA4-4576-ACC8-5675042D7599}']
  { Create an Module Instance upon the Module IID }
  function CreateInstance(ModuleID: TStrGUID): TInstanceID; stdcall;
  { Create an Module Instance upon the Module IID, alternative you can specify the InstanceID to use ( used for Instance Recover at StartUp }
  function CreateInstanceEx(ModuleID: TStrGUID; InstanceID: TInstanceID): TInstanceID; stdcall;
  { Delete the Instance }
  procedure DeleteInstance(InstanceID: TInstanceID); stdcall;
  { Recreates the Interfaces for a specified IID after a Module Reload/Load and assigns it in the private Array }
  procedure ConnectModuleInstances(ModuleID: TStrGUID); stdcall;
  { Frees the Interfaces for a specified IID before a Module Unload }
  procedure DisconnectModuleInstances(ModuleID: TStrGUID); stdcall;
  { Some useful Functions }
  function InstanceExists(InstanceID: TInstanceID): Boolean; stdcall;
  procedure CheckInstance(InstanceID: TInstanceID); stdcall;
  function GetInstance(InstanceID: TInstanceID): TIPSInstance; stdcall;
  function GetInstances(InstanceIDs: TInstanceIDs): TIPSInstances; stdcall;
  function GetInstanceList(): TInstanceIDs; stdcall;
  function GetInstanceListByModuleType(ModuleType: TIPSModuleType): TInstanceIDs; stdcall;
  function GetInstanceListByModuleID(ModuleID: TStrGUID): TInstanceIDs; stdcall;
  function GetInstanceIDByName(Name: String; ParentID: TIPSID): TInstanceID; stdcall;
  { Checks compatibility }
  function IsInstanceCompatible(InstanceID, ParentInstanceID: TInstanceID): Boolean; stdcall;
  function GetCompatibleInstances(InstanceID: TInstanceID): TInstanceIDs; stdcall;
 end;

 IIPSInstanceManagerEx=interface(IIPSMessageSink)
  ['{E359382A-74B9-4BD7-A708-7D16964311FF}']
  { Sets the Instance Status }
  procedure SetInstanceStatus(InstanceID: TInstanceID; InstanceStatus: Integer); stdcall;
 end;

// --- VariableManager Interface
 IIPSVariableManager=interface(IIPSMessageSink)
  ['{5528E859-0D7D-41CE-9E82-D7CF50228A5B}']
  { Created a Variable }
  function CreateVariable(VarType: TIPSVarType): TVariableID; stdcall;
  { Created a Variable }
  function CreateVariableEx(VarType: TIPSVarType; VariableID: TVariableID): TVariableID; stdcall;
  { Deleted a Variable }
  procedure DeleteVariable(VariableID: TVariableID); stdcall;
  { Reads/Writes the Variable Value }
  function ReadVariableBoolean(VariableID: TVariableID): Boolean; stdcall;
  procedure WriteVariableBoolean(VariableID: TVariableID; VarValue: Boolean); stdcall;
  function ReadVariableInteger(VariableID: TVariableID): Integer; stdcall;
  procedure WriteVariableInteger(VariableID: TVariableID; VarValue: Integer); stdcall;
  function ReadVariableFloat(VariableID: TVariableID): Double; stdcall;
  procedure WriteVariableFloat(VariableID: TVariableID; VarValue: Double); stdcall;
  function ReadVariableString(VariableID: TVariableID): String; stdcall;
  procedure WriteVariableString(VariableID: TVariableID; VarValue: String); stdcall;
  function ReadVariableStringEx(VariableID: TVariableID; var Base64Encoded: Boolean): String; stdcall;
  procedure WriteVariableStringEx(VariableID: TVariableID; VarValue: String; Base64Encoded: Boolean); stdcall;
  { Legacy Support - Returns first match only }
  function GetVariableID(VarName: String): TVariableID; stdcall;
  { Get a single Variable }
  function VariableExists(VariableID: TVariableID): Boolean; stdcall;
  procedure CheckVariable(VariableID: TVariableID); stdcall;
  function GetVariable(VariableID: TVariableID): TIPSVariable; stdcall;
  function GetVariables(VariableIDs: TVariableIDs): TIPSVariables; stdcall;
  function GetVariableList(): TVariableIDs; stdcall;
  function GetVariableIDByName(Name: String; ParentID: TIPSID): TVariableID; stdcall;
  { Assign Profile }
  procedure SetVariableCustomProfile(VariableID: TVariableID; VarProfileName: String); stdcall;
  procedure SetVariableCustomAction(VariableID: TVariableID; ScriptID: TScriptID); stdcall;
  { Variable Profile Pool }
  procedure CreateVariableProfile(VarProfileName: String; VarProfileType: TIPSVarType); stdcall;
  procedure DeleteVariableProfile(VarProfileName: String); stdcall;
  { Modify Profile }
  procedure SetVariableProfileText(VarProfileName: String; Prefix, Suffix: String); stdcall;
  procedure SetVariableProfileValues(VarProfileName: String; MinValue, MaxValue, StepSize: Double); stdcall;
  procedure SetVariableProfileDigits(VarProfileName: String; Digits: Integer); stdcall;
  procedure SetVariableProfileAssociation(VarProfileName: String; AssociationValue: Double; AssociationName, AssociationIcon: String); stdcall;
  procedure SetVariableProfileAssociations(VarProfileName: String; Associations: TIPSValueAssociations); stdcall;
  procedure SetVariableProfileIcon(VarProfileName: String; Icon: String); stdcall;
  { Enumeration }
  function VariableProfileExists(VarProfileName: String): Boolean; stdcall;
  procedure CheckVariableProfile(VarProfileName: String); stdcall;
  function GetVariableProfile(VarProfileName: String): TIPSVarProfile; stdcall;
  function GetVariableProfiles(VarProfileNames: TStringArray): TIPSVarProfiles; stdcall;
  function GetVariableProfileList(): TStringArray; stdcall;
  function GetVariableProfileListByType(VarProfileType: TIPSVarType): TStringArray; stdcall;
 end;

 IIPSVariableManagerEx=interface(IIPSMessageSink)
  ['{E731790D-F4DB-4F20-BAF8-B8A9C5F95546}']
  { Set Default Profile }
  procedure SetVariableProfile(VariableID: TVariableID; VarProfileName: String); stdcall;
  { Variable Profile Pool }
  procedure CreateVariableProfileEx(VarProfile: TIPSVarProfile); stdcall;
 end;

// --- ScriptManager Interface
 IIPSScriptManager=interface(IIPSMessageSink)
  ['{ED275B5F-4944-4B37-920F-498C1AFFB14F}']
  { Register / Unregister a Script by providing a Filename which can be found in %KERNELDIR%\scripts }
  function CreateScript(ScriptType: TIPSScriptType): TScriptID; stdcall;
  function CreateScriptEx(ScriptType: TIPSScriptType; ScriptID: TScriptID): TScriptID; stdcall;
  procedure DeleteScript(ScriptID: TScriptID; DeleteFile: Boolean = False); stdcall;
  { Configure }
  procedure SetScriptFile(ScriptID: TScriptID; ScriptFile: String); stdcall;
  { Legacy Support - Returns first match only }
  function GetScriptID(ScriptName: String): TScriptID; stdcall;
  { Get Script Inforamtions }
  function ScriptExists(ScriptID: TScriptID): Boolean; stdcall;
  procedure CheckScript(ScriptID: TScriptID); stdcall;
  function GetScript(ScriptID: TScriptID): TIPSScript; stdcall;
  function GetScripts(ScriptIDs: TScriptIDs): TIPSScripts; stdcall;
  function GetScriptList(): TScriptIDs; stdcall;
  function GetScriptIDByName(Name: String; ParentID: TIPSID): TScriptID; stdcall;
  function GetScriptIDByFile(Filepath: String): TScriptID; stdcall;
 end;

//-- EventManager Interface
 IIPSEventManager = interface(IIPSMessageSink)
  ['{FC71B118-4077-4113-882B-66017F889C26}']
  { Create / Delete Events }
  function CreateEvent(EventType: TIPSEventType): TEventID; stdcall;
  function CreateEventEx(EventType: TIPSEventType; EventID: TEventID): TEventID; stdcall;
  procedure DeleteEvent(EventID: TEventID); stdcall;

  { Configure Events }
  procedure SetEventActive(EventID: TEventID; Active: Boolean); stdcall;
  procedure SetEventLimit(EventID: TEventID; Count: Integer); stdcall;
  procedure SetEventTrigger(EventID: TEventID; TriggerType: TIPSTriggerType; TriggerVariableID: TVariableID); stdcall;
  procedure SetEventTriggerValue(EventID: TEventID; TriggerValue: Variant); stdcall;
  procedure SetEventTriggerSubsequentExecution(EventID: TEventID; AllowSubsequentExecutions: Boolean); stdcall;
  procedure SetEventCyclic(EventID: TEventID; DateType: TIPSCyclicDateType; DateValue, DateDay, DateDayValue: Integer; TimeType: TIPSCyclicTimeType; TimeValue: Integer); stdcall;
  procedure SetEventCyclicDateBounds(EventID: TEventID; FromDate, ToDate: TDateTime); stdcall;
  procedure SetEventCyclicTimeBounds(EventID: TEventID; FromTime, ToTime: TDateTime); stdcall;
  procedure SetEventScript(EventID: TEventID; EventScript: String); stdcall;

  { Retrieve Events }
  function EventExists(EventID: TEventID): Boolean; stdcall;
  procedure CheckEvent(EventID: TEventID); stdcall;
  function GetEvent(EventID: TEventID): TIPSEvent; stdcall;
  function GetEvents(EventIDs: TEventIDs): TIPSEvents; stdcall;
  function GetEventList(): TEventIDs; stdcall;
  function GetEventListByType(EventType: TIPSEventType): TEventIDs; stdcall;
  function GetEventIDByName(Name: String; ParentID: TIPSID): TEventID; stdcall;

  { Retrieve Events for Scripts/Variables }
  function GetVariableEventList(VariableID: TVariableID): TEventIDs; stdcall;
  function GetScriptEventList(ScriptID: TScriptID): TEventIDs; stdcall;
 end;

 // --- MediaManager Interface
 IIPSMediaManager=interface(IIPSMessageSink)
  ['{53392D3E-3AA1-4A1C-B3A3-F39F851D2D63}']
  { Register / Unregister a Media by providing a Filename which can be found in %KERNELDIR%\Medias }
  function CreateMedia(MediaType: TIPSMediaType): TMediaID; stdcall;
  function CreateMediaEx(MediaType: TIPSMediaType; MediaID: TMediaID): TMediaID; stdcall;
  procedure DeleteMedia(MediaID: TMediaID; DeleteFile: Boolean = False); stdcall;
  { Configure }
  procedure SetMediaFile(MediaID: TMediaID; MediaFile: String; FileMustExist: Boolean = True); stdcall;
  { Legacy Support - Returns first match only }
  function GetMediaID(MediaName: String): TMediaID; stdcall;
  { Get Media Informations }
  function MediaExists(MediaID: TMediaID): Boolean; stdcall;
  procedure CheckMedia(MediaID: TMediaID); stdcall;
  function GetMedia(MediaID: TMediaID): TIPSMedia; stdcall;
  function GetMedias(MediaIDs: TMediaIDs): TIPSMedias; stdcall;
  function GetMediaList(): TMediaIDs; stdcall;
  function GetMediaListByType(MediaType: TIPSMediaType): TMediaIDs; stdcall;
  function GetMediaIDByName(Name: String; ParentID: TIPSID): TMediaID; stdcall;
  function GetMediaIDByFile(Filepath: String): TMediaID; stdcall;
  { Actions }
  procedure SendMediaEvent(MediaID: TMediaID); stdcall;
  procedure SendMediaEventEx(MediaID: TMediaID; Action: String); stdcall;
 end;

 // --- LinkManager Interface
 IIPSLinkManager=interface(IIPSMessageSink)
  ['{275A6FAF-EA8B-4641-97CB-31ED5673F2F4}']
  function CreateLink(): TLinkID; stdcall;
  function CreateLinkEx(LinkID: TLinkID): TIPSID; stdcall;
  procedure DeleteLink(LinkID: TLinkID); stdcall;
  { Set the Objects Name/Parent }
  procedure SetLinkChildID(LinkID: TLinkID; ChildID: TIPSID); stdcall;
  { Getter Functions }
  function LinkExists(LinkID: TLinkID): Boolean; stdcall;
  procedure CheckLink(LinkID: TLinkID); stdcall;
  function GetLink(LinkID: TLinkID): TIPSLink; stdcall;
  function GetLinks(LinkIDs: TLinkIDs): TIPSLinks; stdcall;
  function GetLinkList(): TLinkIDs; stdcall;
  function GetLinkIDByName(Name: String; ParentID: TIPSID): TLinkID; stdcall;
 end;

//-- ScriptEngine Interface
 IIPSScriptEngine=interface(IIPSMessageSink)
  ['{CEB34938-A40C-4A5E-94B3-9013772CFB62}']
  { Scripting Functions }
  function GetFunctionList(Filter: TIPSID = 0): TIPSPHPFunctions; stdcall;
  { ScriptThread Status }
  function ScriptThreadExists(ThreadID: TThreadID): Boolean; stdcall;
  procedure CheckScriptThread(ThreadID: TThreadID); stdcall;
  function GetScriptThread(ThreadID: TThreadID): TIPSScriptThreadInfo; stdcall;
  function GetScriptThreads(ThreadIDs: TThreadIDs): TIPSScriptThreadInfos; stdcall;
  function GetScriptThreadList(): TThreadIDs; stdcall;
  { Execution }
  function ExecuteScript(ScriptID: TScriptID; DoWait: Boolean): String; stdcall;
  function ExecuteFile(ScriptFile: String; DoWait: Boolean): String; stdcall;
  function ExecuteText(ScriptText: String; DoWait: Boolean): String; stdcall;
  { Extended Execution }
  procedure ExecuteScriptEx(ScriptID: TScriptID; Sender: String; DoWait: Boolean; var ExecuteInfo: TIPSExecuteInfo); stdcall;
  procedure ExecuteFileEx(ScriptFile: String; Sender: String; DoWait: Boolean; var ExecuteInfo: TIPSExecuteInfo); stdcall;
  procedure ExecuteTextEx(ScriptText: String; Sender: String; DoWait: Boolean; var ExecuteInfo: TIPSExecuteInfo); stdcall;
 end;

 IIPSScriptEngineEx=interface(IIPSMessageSink)
  ['{028ECD1E-544B-4DC2-969A-4858F1584EEE}']
  { Register PHP Functions }
  procedure RegisterFunction(FunctionName: String; Parameters: TIPSPHPParameterInfos; Result: TIPSPHPParameterInfo; FunctionHandler: TIPSPHPExecute; InstanceID: TInstanceID = 0; Singleton: Boolean = True); stdcall;
  procedure UnregisterFunction(FunctionName: String; InstanceID: TInstanceID = 0); stdcall;
  { Register Instance Functions }
  procedure RegisterFunctions(Info: PTypeInfo; Prefix: String; FunctionHandler: TIPSPHPExecute; InstanceID: TInstanceID = 0; Singleton: Boolean = True); stdcall;
  procedure UnregisterFunctions(InstanceID: TInstanceID); stdcall;
 end;

//-- TimerPool Interface
 IIPSTimerPool = interface(IIPSMessageSink)
 ['{BB4A97E0-D803-4D5E-9A6A-1D5DCC49EF47}']
  { Get Timers }
  function TimerExists(TimerID: TTimerID): Boolean; stdcall;
  procedure CheckTimer(TimerID: TTimerID); stdcall;
  function GetTimer(TimerID: TTimerID): TIPSTimer; stdcall;
  function GetTimers(TimerIDs: TTimerIDs): TIPSTimers; stdcall;
  function GetTimerList(): TTimerIDs; stdcall;
 end;

 IIPSTimerPoolEx = interface(IIPSMessageSink)
  ['{6EBEA65A-62D7-4DE8-8FA2-E268E35D5DAA}']
  { Register Timer }
  function RegisterTimer(Name: String; InstanceID: TInstanceID; OnEvent: TTimerEvent; Seconds: Integer = 0): TTimerID; stdcall;
  function RegisterTimerEx(Name: String; InstanceID: TInstanceID; OnEvent: TTimerEvent; Milliseconds: Integer = 0): TTimerID; stdcall;
  procedure UnregisterTimer(TimerID: TTimerID); stdcall;
  { Configure Timer }
  procedure SetInterval(TimerID: TTimerID; Seconds: Integer); stdcall;
  procedure SetIntervalEx(TimerID: TTimerID; Milliseconds: Integer); stdcall;
  procedure ResetTimer(TimerID: TTimerID); stdcall;
 end;

//-- FilePool Interface
 IIPSFilePool = interface(IIPSMessageSink)
  ['{440DD282-CD53-44F3-B739-75B0B1485F9A}']
  function FileExists(Filename: String): Boolean; stdcall;
  procedure DownloadFile(Filename: String; var Data: TSOAPAttachment); stdcall;
  procedure UploadFile(Filename: String; Data: TSOAPAttachment); stdcall;
  procedure RenameFile(Filename, NewFilename: String); stdcall;
  function GetFileListing(Directory: String; Mask: String = '*'): TStringArray; stdcall;
 end;

 IIPSFilePoolEx = interface(IIPSMessageSink)
  ['{5BAE6D93-3D0B-48E0-B2B1-CB176E76E3AB}']
  procedure LockFile(Filename: String; WriteLock: Boolean); stdcall;
  procedure UnlockFile(Filename: String; WriteLock: Boolean); stdcall;
  function IsLocked(Filename: String; WriteLock: Boolean): Boolean; stdcall;
 end;

// --- Settings Interface

 //Rewrite postponed for versions after 2.0
 //Database backend (possibly sqlite) planned.
 IIPSSettings=interface(IIPSMessageSink)
  ['{2058C7E1-D444-4EB1-B67A-FD48DBA7D06F}']
  { Saves all Settings to File, Will also query all Objects to Save }
  function SaveSettings(): HRESULT;
  { Returns an Array of TInstanceID with all available InstanceIDs from Settings.XML }
  function GetInstanceIDs(var Values: TInstanceIDs; var Count: Integer): HRESULT;
  { Saves all connection Points of an Instance }
  function WriteInstanceConnections(InstanceID: TInstanceID; Values: TInstanceIDs; Count: Integer): HRESULT; 
  { Returns an Array of TInstanceID with all Connections ( InstanceID's ) from InstanceID }
  function ReadInstanceConnections(InstanceID: TInstanceID; var Values: TInstanceIDs; var Count: Integer): HRESULT; 
  { Returns an Array of Items in Section }
  function GetInstanceItems(InstanceID: TInstanceID; Section: Array of String; var Values: TStringArray; var Count: Integer): HRESULT; 
  { Read/Write Instance Data, like Instance specific Settings }
  function WriteInstanceBoolean(InstanceID: TInstanceID; Section: Array of String; Name: String; Value: Boolean): HRESULT; 
  function WriteInstanceInteger(InstanceID: TInstanceID; Section: Array of String; Name: String; Value: Integer): HRESULT;
  function WriteInstanceFloat(InstanceID: TInstanceID; Section: Array of String; Name: String; Value: Double): HRESULT; 
  function WriteInstanceString(InstanceID: TInstanceID; Section: Array of String; Name: String; Value: String): HRESULT; 
  function ReadInstanceBoolean(InstanceID: TInstanceID; Section: Array of String; Name: String; var Value: Boolean): HRESULT;
  function ReadInstanceInteger(InstanceID: TInstanceID; Section: Array of String; Name: String; var Value: Integer): HRESULT; 
  function ReadInstanceFloat(InstanceID: TInstanceID; Section: Array of String; Name: String; var Value: Double): HRESULT; 
  function ReadInstanceString(InstanceID: TInstanceID; Section: Array of String; Name: String; var Value: String): HRESULT; 
  { Delete Instance Item }
  function DeleteInstanceItem(InstanceID: TInstanceID; Section: Array of String): HRESULT; 
  { Read/Write direct XML Streams }
  function ReadInstanceXML(InstanceID: TInstanceID; var XMLContent: String): HRESULT; 
  function WriteInstanceXML(InstanceID: TInstanceID; XMLContent: String): HRESULT;
  { Returns an Array of Items in Section }
  function GetItems(Section: Array of String; var Values: TStringArray; var Count: Integer): HRESULT;
  { Read/Write Global Settings }
  function WriteBoolean(Section: Array of String; Name: String; Value: Boolean): HRESULT;
  function WriteInteger(Section: Array of String; Name: String; Value: Integer): HRESULT; 
  function WriteFloat(Section: Array of String; Name: String; Value: Double): HRESULT;
  function WriteString(Section: Array of String; Name: String; Value: String): HRESULT; 
  function ReadBoolean(Section: Array of String; Name: String; var Value: Boolean): HRESULT; 
  function ReadInteger(Section: Array of String; Name: String; var Value: Integer): HRESULT; 
  function ReadFloat(Section: Array of String; Name: String; var Value: Double): HRESULT; 
  function ReadString(Section: Array of String; Name: String; var Value: String): HRESULT; 
  { Delete Item }
  function DeleteItem(Section: Array of String): HRESULT; 
 end;

// --- DebugServer Interface
 IIPSDebugServer = interface(IIPSMessageSink)
 ['{CA3F512F-B1FB-4D2E-94D8-47C4139555B7}']
  procedure SendData(SenderID: TIPSID; Message, Data: String);

  function IsSessionValid(Session: String): Boolean; stdcall;
  function GetNewSessionID: String; stdcall;
  function GetMessages(SessionID: String): TIPSDebugMessages; stdcall;

  procedure AddListener(SessionID: String; ID: TIPSID); stdcall;
  procedure RemoveListener(SessionID: String; ID: TIPSID); stdcall;
 end;

// --- SOAPServer Interface
 IIPSSOAPServer = interface(IIPSMessageSink)
 ['{2FA0A102-A67D-4F7C-BA16-4E5ACC7EF71C}']
  { Get new SessionID to use }
  function IsSessionValid(SessionID: String): Boolean; stdcall;
  function StartSession(FilterType: TIPSFilterType): String; stdcall;
  procedure AddSessionFilter(SessionID: String; SenderID, Message: Integer); stdcall;
  procedure RemoveSessionFilter(SessionID: String; SenderID, Message: Integer); stdcall;
  procedure ClearSessionFilter(SessionID: String); stdcall;
  function GetSessionFilter(SessionID: String): TIPSMessageFilters; stdcall;
  function GetSessionMessages(SessionID: String): TIPSMessages; stdcall;
 end;

//------------------------------------------------------------------------------
// Basic Implementations
//------------------------------------------------------------------------------
// --- Helper Class
 TStrListHelper = class helper for TStrings
 public
  procedure FreeObjects;
  function toArray: TStringArray;
 end;

// --- Helper Class
 TThreadListHelper = class helper for TThreadList
 public
  procedure FreeObjects;
 end;

// --- MessageSink Class
 TIPSMessageObject=class(TInterfacedObject, IIPSMessageSink)
  protected
   fKernel: IIPSKernel;
   fKernelRunlevel: Integer;
   procedure ProcessKernelRunlevelChange(Runlevel: Integer); virtual;
   //Helper
   procedure LogModuleMessage(Str: String; KMsg: Integer = KL_MESSAGE; KID: TIPSID = 0);
   function GetLocation(ID: TIPSID): String;
   function GetName(ID: TIPSID): String;
  public
   constructor Create(Kernel: IIPSKernel); virtual;
   procedure MessageSink(var Msg: TIPSMessage); virtual;
   { Should return 1. Only the Kernel should hold a reference }
   function ReferenceCount(): Integer;
 end;

 TIPSCallableObject=class(TIPSMessageObject)
  public
   class function FilterFunctions(): TStringArray; virtual;
 end;

// --- Module Class
 TBoolSetter = procedure(Value: Boolean) of Object; stdcall;
 TBoolGetter = function: Boolean of Object; stdcall;

 TIntSetter = procedure(Value: Integer) of Object; stdcall;
 TIntGetter = function: Integer of Object; stdcall;

 TFloatSetter = procedure(Value: Double) of Object; stdcall;
 TFloatGetter = function: Double of Object; stdcall;

 TStrSetter = procedure(Value: String) of Object; stdcall;
 TStrGetter = function: String of Object; stdcall;

 TPropertyHolder = class
  public
   PropertyType: TIPSVarType;
   Getter: TMethod;
   Setter: TMethod;
 end;

 //Exception
 EIPSModuleObject = class(Exception);

 //Class
 TIPSModuleObject = class(TIPSCallableObject, IIPSModule)
  private
   { Status Variables }
   fCurrentVarList    : THashedStringList;//<TStatusVarHolder>
   fChangedVarList    : THashedStringList;//<TStatusVarHolder>
   { Properties }
   fProperties        : TStringList;
   { Searching }
   fNextSearchID      : Integer;
   { Settings }
   fHasChanges        : Boolean;
   fIsSearching       : Boolean;
   fSupportsSearching : Boolean;
   procedure AssignStatusVariables; stdcall;
   procedure SendSettingsChangedMessage;
   procedure RegisterProperty(PropertyName: String; PType: TIPSVarType; Getter: TMethod; Setter: TMethod);
  protected
   fKernel          : IIPSKernel;
   fInstanceID      : TInstanceID;
   { Is Remote Call }
   function IsRemoteCall: Boolean;
   { Constructor & Destructor Processing }
   procedure ConstructorPostProcess; virtual;
   procedure DestructorPreProcess; virtual;
   { Easy LogMessages }
   procedure LogMessage(LType: Integer; LMessage: String);
   procedure SendData(Message, Data: String);
   procedure SetStatus(Status: Integer);
   procedure SetSummary(Summary: String);
   { PHP Function Handling }
   procedure RegisterFunctions();
   { Settings Handling }
   procedure RegisterBoolProperty(PropertyName: String; Getter: TBoolGetter; Setter: TBoolSetter);
   procedure RegisterIntProperty(PropertyName: String; Getter: TIntGetter; Setter: TIntSetter);
   procedure RegisterFloatProperty(PropertyName: String; Getter: TFloatGetter; Setter: TFloatSetter);
   procedure RegisterStrProperty(PropertyName: String; Getter: TStrGetter; Setter: TStrSetter);
   { Status Variable Handling }
   procedure RegisterVariable(VarIdent: String; VarName: String; VarType: TIPSVarType; VarProfile: String = ''; VarPosition: Integer = 0);
   procedure UnregisterVariable(VarIdent: String; DeleteVariable: Boolean = False);
   procedure MaintainVariable(VarIdent: String; VarName: String; VarType: TIPSVarType; Keep: Boolean; VarProfile: String = ''; VarPosition: Integer = 0);
   procedure RegisterActionHandler(VarIdent: String; Handler: TIPSActionRequestHandler);
   { Status Variable Profiles }
   procedure RegisterProfile(VarProfile: TIPSVarProfile);
   { Searching Support }
   procedure SetSearchSupport(Enable: Boolean);
   procedure SendSearchUpdate;
   procedure SendSearchProgress(Status: String);
   procedure SendSearchComplete;
   function FoundDevice(Device: String; SearchStatus: Integer; InstanceID: TInstanceID = 0): Integer; stdcall;
   { Device Data Handling }
   function RequireParent(ModuleID: TGUID; ForceCreate: Boolean = False): Boolean;
   function ForceParent(ModuleID: TGUID): Boolean;
   function GetParent(): IIPSModule;
   function GetChildren(): TInterfaceList;
   function HasActiveParent(): Boolean;
   { PHP Event Handling }
   procedure HandlePHPEvent(FunctionName    : String;
                            Parameters      : TIPSPHPParameters;
                            var ReturnValue : TIPSPHPParameter); virtual;
   { Settings Handling }
   procedure SettingsChanged;

   { MessageSink Forwardings }
   procedure ProcessInstanceStatusChange(InstanceID: TInstanceID; Status: Integer); virtual;
  public
   constructor Create(Kernel: IIPSKernel; InstanceID: TInstanceID); reintroduce; virtual;
   destructor Destroy; override;
   { Message Sink }
   procedure MessageSink(var Msg: TIPSMessage); override;
   { Status Variable Handling }
   function StatusVariableExists(VariableIdent: String): Boolean; stdcall;
   function GetStatusVariable(VariableIdent: String): TIPSStatusVariable; stdcall;
   function GetStatusVariables(): TIPSStatusVariables; stdcall;
   function GetStatusVariableIdents(): TStringArray; stdcall;
   procedure SetStatusVariableUseAction(VariableIdent: String; UseAction: Boolean); stdcall;
   function GetStatusVariableID(VariableIdent: String): TVariableID; stdcall;
   procedure SetStatusVariableID(VariableIdent: String; VariableID: TVariableID); stdcall;
   procedure RequestAction(VariableIdent: String; Value: Variant); stdcall;
   { Searching }
   function SupportsSearching: Boolean; virtual; stdcall;
   procedure DoSearching(Enable: Boolean); virtual; stdcall;
   procedure SelectDevice(SearchID: Integer); virtual; stdcall;
   function IsSearching: Boolean; stdcall;
   { Save/Load Settings }
   procedure LoadSettings(); virtual; stdcall;
   procedure SaveSettings(); virtual; stdcall;
   { Process changes }
   function HasChanges: Boolean; stdcall;
   procedure ResetChanges(); virtual; stdcall;
   procedure ApplyChanges(); virtual; stdcall;
   { InstanceID }
   function GetInstanceID(): TInstanceID; stdcall;

   // --- IIPSCallable Implementation
   class function FilterFunctions(): TStringArray; override;

   { Class Functions }
   class function GetModuleID(): TStrGUID; virtual; abstract;
   class function GetModuleType(): TIPSModuleType; virtual; abstract;
   class function GetModuleName(): String; virtual; abstract;
   class function GetParentRequirements(): TStrGUIDs; virtual;
   class function GetChildRequirements(): TStrGUIDs; virtual;
   class function GetImplemented(): TStrGUIDs; virtual;
   class function GetVendor(): String; virtual;
   class function GetAliases(): TStringArray; virtual;
 end;

//------------------------------------------------------------------------------
// Helper Functions
//------------------------------------------------------------------------------
 procedure DisableContext;
 procedure EnableContext;

 function StringToVarType(VarType: String): TIPSVarType;
 function VarTypeToString(VarType: TIPSVarType): String;
 function StringToEventType(EventType: String): TIPSEventType;
 function EventTypeToString(EventType: TIPSEventType): String;
 function StringToTriggerType(TriggerType: String): TIPSTriggerType;
 function TriggerTypeToString(TriggerType: TIPSTriggerType): String; 
 function StringToCyclicDateType(CyclicDateType: String): TIPSCyclicDateType;
 function CyclicDateTypeToString(CyclicDateType: TIPSCyclicDateType): String;
 function StringToCyclicTimeType(CyclicTimeType: String): TIPSCyclicTimeType;
 function CyclicTimeTypeToString(CyclicTimeType: TIPSCyclicTimeType): String;
 function NamedVarType(VName: String; VType: TIPSVarType): TIPSPHPParameterInfo;
 function NamedVarTypes(VInfos: Array of TIPSPHPParameterInfo): TIPSPHPParameterInfos;
 function ValueAssociation(Value: Double; Name, Icon: String): TIPSValueAssociation;
 function ValueAssociations(VInfos: Array of TIPSValueAssociation): TIPSValueAssociations;

 type
  TDefaultProfiles = (dpTemperature, dpHumidity, dpAirPressure, dpIlluminance,
                      dpSwitch, dpIntensity1, dpIntensity100, dpIntensity255,
                      dpAlarm, dpAlarmReversed, dpWindow, dpWindowReversed, dpBattery, dpBatteryReversed,
                      dpBattery100, dpWindSpeedKmh, dpWindSpeedMs, dpWindDirection, dpUVIndex,
                      dpRainfall, dpRaining, dpValve, dpShutter, dpVolt230, dpAmpere16, dpWatt3680,
                      dpElectricity, dpWater, dpGas
);

 function GetDefaultProfile(DefaultProfile: TDefaultProfiles): String;

implementation

//------------------------------------------------------------------------------
procedure DisableContext;
begin
 DataContextDisabled := True;
end;

//------------------------------------------------------------------------------
procedure EnableContext;
begin
 DataContextDisabled := False;
end;

//------------------------------------------------------------------------------
function NamedVarType(VName: String; VType: TIPSVarType): TIPSPHPParameterInfo;
begin

 DisableContext;
 try
  Result := TIPSPHPParameterInfo.Create;
 finally
  EnableContext;
 end;
 Result.Description := VName;
 Result.Type_ := VType;

end;

//------------------------------------------------------------------------------
function NamedVarTypes(VInfos: Array of TIPSPHPParameterInfo): TIPSPHPParameterInfos;
var i: Integer;
begin

 SetLength(Result, High(VInfos)+1);
 for i := Low(VInfos) to High(VInfos) do
  Result[i] := VInfos[i];

end;

//------------------------------------------------------------------------------
function ValueAssociation(Value: Double; Name, Icon: String): TIPSValueAssociation;
begin

 DisableContext;
 try
  Result := TIPSValueAssociation.Create;
 finally
  EnableContext;
 end;
 Result.Value := Value;
 Result.Name := Name;
 Result.Icon := Icon;

end;

//------------------------------------------------------------------------------
function ValueAssociations(VInfos: Array of TIPSValueAssociation): TIPSValueAssociations;
var i: Integer;
begin

 SetLength(Result, High(VInfos)+1);
 for i := Low(VInfos) to High(VInfos) do
  Result[i] := VInfos[i];
  
end;

//------------------------------------------------------------------------------
function GetDefaultProfile(DefaultProfile: TDefaultProfiles): String;
begin

 case DefaultProfile of
  dpTemperature     : Result := 'Temperature';
  dpHumidity        : Result := 'Humidity';
  dpAirPressure     : Result := 'AirPressure';
  dpIlluminance     : Result := 'Illumination';
  dpSwitch          : Result := 'Switch';
  dpIntensity1      : Result := 'Intensity.1';
  dpIntensity100    : Result := 'Intensity.100';
  dpIntensity255    : Result := 'Intensity.255';
  dpAlarm           : Result := 'Alert';
  dpAlarmReversed   : Result := 'Alert.Reversed';
  dpWindow          : Result := 'Window';
  dpWindowReversed  : Result := 'Window.Reversed';
  dpBattery         : Result := 'Battery';
  dpBatteryReversed : Result := 'Battery.Reversed';
  dpBattery100      : Result := 'Battery.100';
  dpWindSpeedKmh    : Result := 'WindSpeed.kmh';
  dpWindSpeedMs     : Result := 'WindSpeed.ms';
  dpWindDirection   : Result := 'WindDirection';
  dpUVIndex         : Result := 'UVIndex';
  dpRainfall        : Result := 'Rainfall';
  dpRaining         : Result := 'Raining';
  dpValve           : Result := 'Valve';
  dpShutter         : Result := 'Shutter';
  dpVolt230         : Result := 'Volt.230';
  dpAmpere16        : Result := 'Ampere.16';
  dpWatt3680        : Result := 'Watt.3680';
  dpElectricity     : Result := 'Electricity';
  dpWater           : Result := 'Water';
  dpGas             : Result := 'Gas';
 else
  Result := '';
 end

end;

//------------------------------------------------------------------------------
function ElementOfConst(aIndex: Integer; aArray: Array Of Const): Variant;
Begin
  With aArray[aIndex] Do
    Case VType Of
      System.vtInteger:
        Result := VInteger;
      System.vtBoolean:
        Result := vBoolean;
      System.vtChar:
        Result := VChar;
      System.vtExtended:
        Result := vExtended^;
      System.vtString:
        Result := VString^;
      System.vtPChar:
        Result := StrPas(VPChar);
      System.vtObject:
        Result := VObject.ClassName;
      System.vtClass:
        Result := VClass.ClassName;
      System.vtAnsiString:
        Result := String(VAnsiString);
      System.vtCurrency:
        Result := VCurrency^;
      System.vtVariant:
        Result := VVariant^;
      System.vtInt64:
        Result := VInt64^;
    End;
End;

//Implements HelperClass
//------------------------------------------------------------------------------
procedure TStrListHelper.FreeObjects;
var i: Integer;
begin

 if Self.Count > 0 then
  for i:=0 to Self.Count-1 do
   Self.Objects[i].Free;

end;

//------------------------------------------------------------------------------
function TStrListHelper.toArray: TStringArray;
var i: Integer;
begin

  SetLength(Result, Count);
  if Count > 0 then
   for i:=0 to Count-1 do
    Result[i] := Self[i];

end;

//------------------------------------------------------------------------------
procedure TThreadListHelper.FreeObjects;
var i: Integer;
    L: TList;
begin

 L := Self.LockList;
 try
  if L.Count > 0 then
   for i:=0 to L.Count-1 do
    TObject(L[i]).Free;

 finally
  Self.UnlockList;
 end;

end;

//------------------------------------------------------------------------------
constructor TIPSRemotable.Create;
begin
 if not DataContextDisabled then
  inherited;
end;

//------------------------------------------------------------------------------
constructor TIPSMessage.Create(SenderID: TIPSID; Message: Integer; Data: Array of Const);
var i: Integer;
begin

 FTimeStamp := Now;
 FSenderID := SenderID;
 FMessage := Message;
 FData := VarArrayCreate([0, High(Data)+1], varVariant);

 for i := Low(Data) to High(Data) do
  FData[i] := ElementOfConst(i, Data);

 inherited Create();

end;

//------------------------------------------------------------------------------
constructor TIPSMessage.Create(Msg: TIPSMessage);
begin

 FTimeStamp := Msg.TimeStamp;
 FSenderID := Msg.SenderID;
 FMessage := Msg.FMessage;
 FData := Msg.FData;

 inherited Create();

end;

//------------------------------------------------------------------------------
procedure TIPSStatusVariable.SetActionHandler(Value: TIPSActionRequestHandler);
begin

 FActionHandler := Value;
 FHasAction := Assigned(FActionHandler);

end;

//------------------------------------------------------------------------------
constructor TIPSDebugMessage.Create(InstanceID: TInstanceID; Message, Data: String);
begin

 FTimeStamp := Now;
 FInstanceID := InstanceID;
 FMessage := Message;
 FData := Data;

 inherited Create();

end;

//------------------------------------------------------------------------------
constructor TIPSDebugMessage.Create(Msg: TIPSDebugMessage);
begin

 FTimeStamp := Msg.TimeStamp;
 FMessage := Msg.FMessage;
 FInstanceID := Msg.FInstanceID;
 FData := Msg.FData;

 inherited Create();

end;

//------------------------------------------------------------------------------
constructor TIPSMessageFilter.Create(SenderID, Message: Integer);
begin

 FSenderID := SenderID;
 FMessage := Message;

 inherited Create();

end;

//Implements IPSInstance
//------------------------------------------------------------------------------
constructor TIPSInstance.Create;
begin
 FModuleInfo := TIPSBasicModuleInformation.Create;
 inherited;
end;

//------------------------------------------------------------------------------
destructor TIPSInstance.Destroy;
begin
 if Self.DataContext = nil then
  FreeAndNil(FModuleInfo);
 inherited;
end;

//Implements IPSVarValue
//------------------------------------------------------------------------------
constructor TIPSVarValue.Create;
begin
 FValueIndex := TIPSVarIndex.Create;
 SetLength(FValueArray, 0);
 inherited;
end;

//------------------------------------------------------------------------------
destructor TIPSVarValue.Destroy;
var i: Integer;
begin
 if Self.DataContext = nil then
  begin
   FValueIndex.Free;
   for i:=Low(FValueArray) to High(FValueArray) do
    FValueArray[i].Free;
  end;
 inherited;
end;

//------------------------------------------------------------------------------
function TIPSVarValue.PrepareAndCheckArrayEx(IndexType: TIPSVarIndexType; Name: String; Index: Integer): Integer;

 function GetNextIndex(): Integer;
 var i: Integer;
 begin

  Result := 0;
  if Length(FValueArray) = 1 then
   exit;

  for i:=0 to Length(FValueArray)-1 do //Do not check last item
   Result := Max(Result, FValueArray[i].ValueIndex.IndexInt);
  Inc(Result);

 end;

begin

 if FValueType <> vtArray then
  raise Exception.Create(_('Variable is not from Array Type'));

 //Create Array if not already there
 if High(FValueArray) <> -1 then
  begin
   //Check Arraytype
   if FValueArray[0].ValueIndex.IndexType <> IndexType then
    raise Exception.Create(_('Cannot add Named Value in Indexed Array'));
  end;

 //Add Slot
 Result := Length(FValueArray);
 SetLength( FValueArray, Result+1 );
 FValueArray[Result] := TIPSVarValue.Create;
 FValueArray[Result].ValueIndex.IndexType := IndexType;

 case IndexType of
   itInt: if Index = -1 then
           FValueArray[Result].ValueIndex.IndexInt := GetNextIndex()
          else
           FValueArray[Result].ValueIndex.IndexInt := Index;
   itStr: FValueArray[Result].ValueIndex.IndexStr := Name;
 end;

end;

//------------------------------------------------------------------------------
function TIPSVarValue.PrepareAndCheckArrayStr(Name: String = ''): Integer;
begin
 Result := PrepareAndCheckArrayEx(itStr, Name, 0);
end;

//------------------------------------------------------------------------------
function TIPSVarValue.PrepareAndCheckArrayInt(Index: Integer = -1): Integer;
begin
 Result := PrepareAndCheckArrayEx(itInt, '', Index);
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddIndexedBool(Value: Boolean; Index: Integer = -1);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayInt( Index);
 //Add Value
 FValueArray[Pos].ValueType := vtBoolean;
 FValueArray[Pos].ValueBoolean := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddIndexedInt(Value: Integer; Index: Integer = -1);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayInt( Index);
 //Add Value
 FValueArray[Pos].ValueType := vtInteger;
 FValueArray[Pos].ValueInteger := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddIndexedFloat(Value: Double; Index: Integer = -1);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayInt(Index);
 //Add Value
 FValueArray[Pos].ValueType := vtFloat;
 FValueArray[Pos].ValueFloat := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddIndexedStr(Value: String; Index: Integer = -1);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayInt( Index);
 //Add Value
 FValueArray[Pos].ValueType := vtString;
 FValueArray[Pos].ValueString := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddIndexedVariant(Value: Variant; Index: Integer = -1);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayInt( Index);
 //Add Value
 FValueArray[Pos].ValueType := vtVariant;
 FValueArray[Pos].ValueVariant := Value;
end;

//------------------------------------------------------------------------------
function TIPSVarValue.AddIndexedArray(Index: Integer = -1): TIPSVarValue;
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayInt(Index);
 //Add Value
 FValueArray[Pos].ValueType := vtArray;
 FValueArray[Pos].ValueArray := NIL;
 Result := FValueArray[Pos];
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddNamedBool(Name: String; Value: Boolean);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayStr(Name);
 //Add Value
 FValueArray[Pos].ValueType := vtBoolean;
 FValueArray[Pos].ValueBoolean := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddNamedInt(Name: String; Value: Integer);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayStr(Name);
 //Add Value
 FValueArray[Pos].ValueType := vtInteger;
 FValueArray[Pos].ValueInteger := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddNamedFloat(Name: String; Value: Double);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayStr(Name);
 //Add Value
 FValueArray[Pos].ValueType := vtFloat;
 FValueArray[Pos].ValueFloat := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddNamedStr(Name: String; Value: String);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayStr(Name);
 //Add Value
 FValueArray[Pos].ValueType := vtString;
 FValueArray[Pos].ValueString := Value;
end;

//------------------------------------------------------------------------------
procedure TIPSVarValue.AddNamedVariant(Name: String; Value: Variant);
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayStr(Name);
 //Add Value
 FValueArray[Pos].ValueType := vtVariant;
 FValueArray[Pos].ValueVariant := Value;
end;

//------------------------------------------------------------------------------
function TIPSVarValue.AddNamedArray(Name: String): TIPSVarValue;
var Pos: Integer;
begin
 Pos := PrepareAndCheckArrayStr(Name);
 //Add Value
 FValueArray[Pos].ValueType := vtArray;
 FValueArray[Pos].ValueArray := NIL;
 Result := FValueArray[Pos];
end;

//Implements TIPSVarProfile
//------------------------------------------------------------------------------
destructor TIPSVarProfile.Destroy;
var i: Integer;
begin

 if Self.DataContext = nil then
  begin
   for i := 0 to High(FAssociations) do
    FAssociations[i].Free;
  end;

 inherited; 

end;

//------------------------------------------------------------------------------
constructor TIPSVarProfile.CreateStringProfile(Name, Icon, Prefix, Suffix: String);
begin

 FProfileType := vtString;
 FIcon := Icon;
 FProfileName := Name;
 FPrefix := Prefix;
 FSuffix := Suffix;

 inherited Create;

end;

//------------------------------------------------------------------------------
constructor TIPSVarProfile.CreateBooleanProfile(Name, Icon, TrueName, TrueIcon, FalseName, FalseIcon: String);
begin

 FMinValue := 0;
 FMaxValue := 1;

 CreateStringProfile(Name, Icon, '', '');

 SetLength(FAssociations, 2);
 FAssociations[0] := ValueAssociation(0, FalseName, FalseIcon);
 FAssociations[1] := ValueAssociation(1, TrueName, TrueIcon);

 FProfileType := vtBoolean;

end;

//------------------------------------------------------------------------------
constructor TIPSVarProfile.CreateIntegerProfile(Name, Icon, Prefix, Suffix: String; MinValue, MaxValue, StepSize: Integer);
begin

 FMinValue := MinValue;
 FMaxValue := MaxValue;
 FStepSize := StepSize;

 CreateStringProfile(Name, Icon, Prefix, Suffix);

 FProfileType := vtInteger;

end;

//------------------------------------------------------------------------------
constructor TIPSVarProfile.CreateIntegerProfileEx(Name, Icon, Prefix, Suffix: String; Associations: TIPSValueAssociations);
begin

 FDigits := 0;
 FAssociations := Associations;

 CreateIntegerProfile(Name, Icon, Prefix, Suffix, Round(Associations[0].Value), Round(Associations[High(Associations)].Value), 0);

end;

//------------------------------------------------------------------------------
constructor TIPSVarProfile.CreateFloatProfile(Name, Icon, Prefix, Suffix: String; MinValue, MaxValue, StepSize: Double; Digits: Integer);
begin

 FDigits := Digits;
 FMinValue := MinValue;
 FMaxValue := MaxValue;
 FStepSize := StepSize;

 CreateStringProfile(Name, Icon, Prefix, Suffix);

 FProfileType := vtFloat;

end;

//------------------------------------------------------------------------------
constructor TIPSVarProfile.CreateFloatProfileEx(Name, Icon, Prefix, Suffix: String; Digits: Integer; Associations: TIPSValueAssociations);
begin

 FAssociations := Associations;

 CreateFloatProfile(Name, Icon, Prefix, Suffix, Associations[0].Value, Associations[High(Associations)].Value, 0, Digits);

end;

//Implements TIPSVariable
//------------------------------------------------------------------------------
constructor TIPSVariable.Create;
begin
 FVariableValue := TIPSVarValue.Create;
 inherited;
end;

//------------------------------------------------------------------------------
destructor TIPSVariable.Destroy;
begin
 if Self.DataContext = nil then
  begin
   FVariableValue.Free;
  end;
 inherited;
end;

//Implements TIPSVariable
//------------------------------------------------------------------------------
constructor TIPSPHPFunction.Create;
begin
 FResult := TIPSPHPParameterInfo.Create;
 inherited;
end;

//------------------------------------------------------------------------------
destructor TIPSPHPFunction.Destroy;
var i: Integer;
begin
 if Self.DataContext = nil then
  begin
   FResult.Free;
   for i:=Low(FParameters) to High(FParameters) do
    FParameters[i].Free;
   for i:=Low(FPHPInstances) to High(FPHPInstances) do
    FPHPInstances[i].Free;
  end;
 inherited;
end;

//------------------------------------------------------------------------------
function TIPSPHPFunction.GetParameters : TIPSPHPParameterInfos;
begin
 Result := FParameters;
end;

//------------------------------------------------------------------------------
procedure TIPSPHPFunction.SetParameters(Value: TIPSPHPParameterInfos);
begin
 FParameters := Value;
end;

//------------------------------------------------------------------------------
function TIPSPHPFunction.GetInstances: TIPSPHPInstances;
begin
 Result := FPHPInstances;
end;

//------------------------------------------------------------------------------
procedure TIPSPHPFunction.SetInstances(Value: TIPSPHPInstances);
begin
 FPHPInstances := Value;
end;

//Implements TIPSPHPVariable
//------------------------------------------------------------------------------
constructor TIPSPHPVariable.Create;
begin
 FVariableValue := TIPSVarValue.Create;
 inherited;
end;

//------------------------------------------------------------------------------
constructor TIPSPHPVariable.CreateNamed(Name: String);
begin
 FVariableName := Name;
 Create;
end;

//------------------------------------------------------------------------------
constructor TIPSPHPVariable.CreateBool(Name: String; Value: Boolean);
begin
 CreateNamed(Name);
 FVariableValue.ValueType := vtBoolean;
 FVariableValue.ValueBoolean := Value;
end;

//------------------------------------------------------------------------------
constructor TIPSPHPVariable.CreateInt(Name: String; Value: Integer);
begin
 CreateNamed(Name);
 FVariableValue.ValueType := vtInteger;
 FVariableValue.ValueInteger := Value;
end;

//------------------------------------------------------------------------------
constructor TIPSPHPVariable.CreateFloat(Name: String; Value: Double);
begin
 CreateNamed(Name);
 FVariableValue.ValueType := vtFloat;
 FVariableValue.ValueFloat := Value;
end;

//------------------------------------------------------------------------------
constructor TIPSPHPVariable.CreateStr(Name: String; Value: String);
begin
 CreateNamed(Name);
 FVariableValue.ValueType := vtString;
 FVariableValue.ValueString:= Value;
end;


//------------------------------------------------------------------------------
destructor TIPSPHPVariable.Destroy;
begin
 if Self.DataContext = nil then
  FVariableValue.Free;
 inherited;
end;

//Implements TIPSExecuteInfo
//------------------------------------------------------------------------------
constructor TIPSExecuteInfo.Create;
begin
 FRequestMethod := rmGET;
 FQueryString := '';
 SetLength(FServerVariables, 0);
 FCookies := '';
 FScriptResult := '';
 SetLength(FScriptHeaders, 0);
 FOnNewContent := nil;
 FData := nil;
 FPostData := TMemoryStream.Create;
 FContentType := '';
 FContentLength := 0;
 inherited;
end;

//------------------------------------------------------------------------------
destructor TIPSExecuteInfo.Destroy;
var i: Integer;
begin
 if Self.DataContext = nil then
  begin
   for i:=Low(FServerVariables) to High(FServerVariables) do
    FServerVariables[i].Free;
   SetLength(FServerVariables, 0);
   SetLength(FScriptHeaders, 0);
   FPostData.Free;
  end;
 inherited;
end;

//Implements MessageObject
//------------------------------------------------------------------------------
constructor TIPSMessageObject.Create(Kernel: IIPSKernel); 
begin

 fKernel := Kernel;
 fKernelRunlevel := Kernel.GetKernelRunlevel;

end;

//------------------------------------------------------------------------------
procedure TIPSMessageObject.MessageSink(var Msg: TIPSMessage);
begin

 case Msg.Message of
  IPS_KERNELMESSAGE:
   begin
    fKernelRunlevel := Msg.Data[0];
    ProcessKernelRunlevelChange(fKernelRunlevel);
   end;
 end;

end;

//------------------------------------------------------------------------------
function TIPSMessageObject.ReferenceCount(): Integer;
begin
 Result := FRefCount;
end;

//------------------------------------------------------------------------------
procedure TIPSMessageObject.LogModuleMessage(Str: String; KMsg: Integer = KL_MESSAGE; KID: TIPSID = 0);
begin

 fKernel.LogMessage(KMsg, KID, MidStr(ClassName, 5, Length(ClassName)), Str);

end;

//------------------------------------------------------------------------------
function TIPSMessageObject.GetLocation(ID: TIPSID): String;
begin
 Result := fKernel.ObjectManager.GetLocation(ID);
end;

//------------------------------------------------------------------------------
function TIPSMessageObject.GetName(ID: TIPSID): String;
begin
 Result := fKernel.ObjectManager.GetName(ID);
end;

//------------------------------------------------------------------------------
procedure TIPSMessageObject.ProcessKernelRunlevelChange(Runlevel: Integer);
begin
 //Nothing to do
end;

//Implements CallableObject
//------------------------------------------------------------------------------
class function TIPSCallableObject.FilterFunctions(): TStringArray;
begin
 SetLength(Result, 0);
end;

//Implements ModuleObject
//------------------------------------------------------------------------------
class function TIPSModuleObject.FilterFunctions(): TStringArray;
begin

 SetLength(Result, 9);
 Result[0] := 'LoadSettings';
 Result[1] := 'SaveSettings';
 Result[2] := 'GetStatusVariables';
 Result[3] := 'SupportsSearching';
 Result[4] := 'DoSearching';
 Result[5] := 'SelectDevice';
 Result[6] := 'IsSearching';
 Result[7] := 'SetStatusVariableID';
 Result[8] := 'GetInstanceID';

end;

//------------------------------------------------------------------------------
constructor TIPSModuleObject.Create(Kernel: IIPSKernel; InstanceID: TInstanceID);
begin

 fKernel := Kernel;
 fInstanceID := InstanceID;
 fSupportsSearching := False;
 fNextSearchID := 0;
 
 inherited Create(Kernel);

 LogMessage(KL_MESSAGE, _('Creating...'));

 fCurrentVarList := THashedStringList.Create;
 fChangedVarList := THashedStringList.Create;
 fProperties     := TStringList.Create;

 RegisterFunctions();

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.IsRemoteCall: Boolean;
begin

 Result := (not DataContextDisabled) and fKernel.IsRemoteCall();

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.ConstructorPostProcess;
begin

 //Try loading from Settings
 LoadSettings;

 try
  ApplyChanges;
 except
  on E: Exception do
   LogMessage(KL_ERROR, E.Message);
 end;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.DestructorPreProcess;
begin

 LogMessage(KL_MESSAGE, _('Deleting...'));

 //Save Settings
 ResetChanges;
 SaveSettings;

end;

//------------------------------------------------------------------------------
destructor TIPSModuleObject.Destroy;
begin

 fKernel.ScriptEngineEx.UnregisterFunctions(fInstanceID);

 //Free Status Variables
 fCurrentVarList.FreeObjects;
 fCurrentVarList.Free;
 fChangedVarList.FreeObjects;
 fChangedVarList.Free;

 //Free Properties
 fProperties.FreeObjects;
 fProperties.Free;

 //Cleanup Interface links
 fKernel := NIL;

 inherited;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.MessageSink(var Msg: TIPSMessage);
begin

 inherited;

 case Msg.Message of
  IM_CHANGESTATUS:
   ProcessInstanceStatusChange(Msg.SenderID, Msg.Data[0]);
 end;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.LogMessage(LType: Integer; LMessage: String);
begin

 fKernel.LogMessage(LType,
                    fInstanceID,
                    Self.GetModuleName(),
                    LMessage);
                    
end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SendData(Message, Data: String);
begin

 fKernel.DebugServer.SendData(fInstanceID, Message, Data);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SetStatus(Status: Integer);
begin

 fKernel.InstanceManagerEx.SetInstanceStatus(fInstanceID, Status);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SetSummary(Summary: String);
begin

 fKernel.ObjectManager.SetSummary(fInstanceID, Summary);

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.StatusVariableExists(VariableIdent: String): Boolean;
begin

 Result := fChangedVarList.IndexOf(VariableIdent) <> -1;

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.GetStatusVariable(VariableIdent: String): TIPSStatusVariable; 
var VarIndex: Integer;
begin

 VarIndex := fChangedVarList.IndexOf(VariableIdent);

 if VarIndex = -1 then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" does not exist'), [VariableIdent]));

 Result := TIPSStatusVariable.Create;
 CopyObject(TIPSStatusVariable(fChangedVarList.Objects[VarIndex]), Result);

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.GetStatusVariables(): TIPSStatusVariables; stdcall;
var i: Integer;
begin

 SetLength(Result, fChangedVarList.Count);
 if fChangedVarList.Count > 0 then
  for i:=0 to fChangedVarList.Count-1 do
   begin
    Result[i] := TIPSStatusVariable.Create;
    CopyObject(TIPSStatusVariable(fChangedVarList.Objects[i]), Result[i]);
   end;

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.GetStatusVariableIdents(): TStringArray;
var i: Integer;
begin

 SetLength(Result, fChangedVarList.Count);
 if fChangedVarList.Count > 0 then
  for i:=0 to fChangedVarList.Count-1 do
   Result[i] := TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SetStatusVariableUseAction(VariableIdent: String; UseAction: Boolean); stdcall;
var VarIndex: Integer;
begin

 VarIndex := fChangedVarList.IndexOf(VariableIdent);

 if VarIndex = -1 then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" does not exist'), [VariableIdent]));

 TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).VariableUseAction := UseAction;

 //Send Change Message
 SettingsChanged;
  
end;

//------------------------------------------------------------------------------
function TIPSModuleObject.GetStatusVariableID(VariableIdent: String): TVariableID;
var VarIndex: Integer;
begin

 VarIndex := fChangedVarList.IndexOf(VariableIdent);

 if VarIndex = -1 then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" does not exist'), [VariableIdent]));

 Result := TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).VariableID;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SetStatusVariableID(VariableIdent: String; VariableID: TVariableID); 
var VarIndex: Integer;
    Variable: TIPSVariable;
begin

 VarIndex := fChangedVarList.IndexOf(VariableIdent);

 if VarIndex = -1 then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" does not exist'), [VariableIdent]));

 //Get Variable
 if VariableID > 0 then
  begin
   Variable := fKernel.VariableManager.GetVariable(VariableID);
   try
    if Variable.VariableValue.ValueType <> TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).VariableType then
     raise EIPSModuleObject.Create(Format(_('VariableType of StatusVariable "%s" and Variable #%d do not match'), [VariableIdent, VariableID]));
   finally
    Variable.Free;
   end;
  end;

 TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).VariableID := VariableID;

 //Send Change Message
 SettingsChanged;

end;

//------------------------------------------------------------------------------c
procedure TIPSModuleObject.RequestAction(VariableIdent: String; Value: Variant); stdcall;
var VarIndex: Integer;
begin

 VarIndex := fChangedVarList.IndexOf(VariableIdent);

 if VarIndex = -1 then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" does not exist'), [VariableIdent]));

 if not Assigned(TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).ActionHandler) then
  raise EIPSModuleObject.Create(Format(_('No action for StatusVariable "%s" definied'), [VariableIdent]));

 TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).ActionHandler(TIPSStatusVariable(fChangedVarList.Objects[VarIndex]), Value);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.AssignStatusVariables;
var i: Integer;
    StatusVar: TIPSStatusVariable;
    Variable: TIPSVariable;
    Changes: Boolean;
    Obj: TIPSObject;
begin

 Changes := False;
 if fChangedVarList.Count > 0 then
  begin
   for i:=0 to fChangedVarList.Count-1 do
    begin
     StatusVar := TIPSStatusVariable(fChangedVarList.Objects[i]);

     if StatusVar.VariableID > 0 then
      if not fKernel.VariableManager.VariableExists(StatusVar.VariableID) then
       StatusVar.VariableID := 0
      else
       begin
        Variable := fKernel.VariableManager.GetVariable(StatusVar.VariableID);
        try
         if Variable.VariableValue.ValueType <> StatusVar.VariableType then
          StatusVar.VariableID := 0;
        finally
         Variable.Free;
        end;
       end;

     if StatusVar.VariableID = 0 then
      begin
       Changes := True;
       StatusVar.VariableID := fKernel.VariableManager.CreateVariable(StatusVar.VariableType);
       fKernel.ObjectManager.SetParent(StatusVar.VariableID, fInstanceID);
       fKernel.ObjectManager.SetName(StatusVar.VariableID, StatusVar.VariableName);
      end;

     //Set Profile everytime
     if not SameText(StatusVar.VariableProfile, '') then
      fKernel.VariableManagerEx.SetVariableProfile(StatusVar.VariableID, StatusVar.VariableProfile);

     //Set Position if new Position is > 0 and the current Position is = 0
     if StatusVar.VariablePosition > 0 then
      begin
       Obj := fKernel.ObjectManager.GetObject(StatusVar.VariableID);
       try
        if Obj.ObjectPosition = 0 then
         fKernel.ObjectManager.SetPosition(StatusVar.VariableID, StatusVar.VariablePosition);
       finally
        Obj.Free;
       end;
      end;

     //Set ReadOnly Flag everytime
     fKernel.ObjectManager.SetReadOnly(StatusVar.VariableID, True);
    end;
  end;

 if Changes then
  SettingsChanged;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SetSearchSupport(Enable: Boolean);
begin

 fSupportsSearching := Enable;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SendSearchUpdate;
begin

 fKernel.PostMessage(fInstanceID, IM_SEARCHUPDATE, [Null]);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SendSearchProgress(Status: String);
begin

 fKernel.PostMessage(fInstanceID, IM_SEARCHPROGRESS, [Status]);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SendSearchComplete;
begin

 fKernel.PostMessage(fInstanceID, IM_SEARCHCOMPLETE, [Null]);

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.FoundDevice(Device: String; SearchStatus: Integer; InstanceID: TInstanceID = 0): Integer; stdcall;
begin

 Result := fNextSearchID;
 fKernel.PostMessage(fInstanceID, IM_SEARCHUPDATE, [fNextSearchID, Device, SearchStatus, InstanceID]);
 Inc(fNextSearchID);

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.SupportsSearching: Boolean;
begin

 Result := fSupportsSearching;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.DoSearching(Enable: Boolean);
begin

 fIsSearching := Enable;
 fKernel.PostMessage(fInstanceID, IM_CHANGESEARCH, [Enable]);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SelectDevice(SearchID: Integer);
begin

 raise EIPSModuleObject.Create(_('This module does not support selective seaching!'));

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.IsSearching: Boolean;
begin

 Result := fIsSearching;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.HandlePHPEvent(FunctionName    : String;
                                          Parameters      : TIPSPHPParameters;
                                          var ReturnValue : TIPSPHPParameter);
begin

 raise EIPSModuleObject.Create(Format(_('PHP function was not handled: %s'), [FunctionName]));

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.LoadSettings();
var i, VariableID: Integer;
    ValBool: Boolean;
    ValInt: Integer;
    ValStr: String;
    ValFloat: Double;
    VarName: String;
    UseAction: Boolean;
begin

 DisableContext;
 try
  //Load Module Settings
  if fProperties.Count > 0 then
   for i:=0 to fProperties.Count-1 do
    begin
     case TPropertyHolder(fProperties.Objects[i]).PropertyType of
      vtBoolean: if not FAILED(fKernel.Settings.ReadInstanceBoolean(fInstanceID, ['Settings'], fProperties[i], ValBool)) then
                  TBoolSetter(TPropertyHolder(fProperties.Objects[i]).Setter)(ValBool);
      vtInteger: if not FAILED(fKernel.Settings.ReadInstanceInteger(fInstanceID, ['Settings'], fProperties[i], ValInt)) then
                  TIntSetter(TPropertyHolder(fProperties.Objects[i]).Setter)(ValInt);
      vtFloat:   if not FAILED(fKernel.Settings.ReadInstanceFloat(fInstanceID, ['Settings'], fProperties[i], ValFloat)) then
                  TFloatSetter(TPropertyHolder(fProperties.Objects[i]).Setter)(ValFloat);
      vtString:  if not FAILED(fKernel.Settings.ReadInstanceString(fInstanceID, ['Settings'], fProperties[i], ValStr)) then
                  TStrSetter(TPropertyHolder(fProperties.Objects[i]).Setter)(ValStr);
     else
      raise EIPSModuleObject.Create(Format(_('Cannot load unsupported Property: %s'), [fProperties[i]]));
     end;
    end;

  //Try to import Variable Data
  if fChangedVarList.Count > 0 then
   for i:=0 to fChangedVarList.Count-1 do
    if not FAILED(fKernel.Settings.ReadInstanceString(fInstanceID, ['Settings'], TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, VarName)) then
     if not SameText(Trim(VarName), '') then
      try
       try
        VariableID := fKernel.VariableManager.GetVariableID(VarName);

        SetStatusVariableID(TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, VariableID);

        //Set StatusVarFlag
        fKernel.ObjectManager.SetReadOnly(VariableID, True);

        //Update Parent
        fKernel.ObjectManager.SetParent(VariableID, fInstanceID);
       except
        on E: Exception do
         LogMessage(KL_ERROR, E.Message);
       end;
      finally
       fKernel.Settings.DeleteInstanceItem(fInstanceID, ['Settings', TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent]);
      end;

  //Load Status Variable Data
  if fChangedVarList.Count > 0 then
   for i:=0 to fChangedVarList.Count-1 do
    begin
     if not FAILED(fKernel.Settings.ReadInstanceInteger(fInstanceID, ['Settings', 'StatusVariables'], TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, VariableID)) then
      try
       SetStatusVariableID(TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, VariableID);

       //Set StatusVarFlag
       fKernel.ObjectManager.SetReadOnly(VariableID, True);
      except
       on E: Exception do
        LogMessage(KL_ERROR, E.Message);
      end;

     try
      if not FAILED(fKernel.Settings.ReadInstanceBoolean(fInstanceID, ['Settings', 'StatusVariablesActions'], TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, UseAction)) then
       SetStatusVariableUseAction(TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, UseAction)
      else
       SetStatusVariableUseAction(TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, true);
     except
      on E: Exception do
       LogMessage(KL_ERROR, E.Message);
     end;

    end;

 finally
  EnableContext;
 end;
   
end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SaveSettings(); 
var i: Integer;
begin

 DisableContext;
 try
  //Save Status Variable Data
  if fChangedVarList.Count > 0 then
   for i:=0 to fChangedVarList.Count-1 do
    begin
     fKernel.Settings.WriteInstanceInteger(fInstanceID, ['Settings', 'StatusVariables'], TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, TIPSStatusVariable(fChangedVarList.Objects[i]).VariableID);
     fKernel.Settings.WriteInstanceBoolean(fInstanceID, ['Settings', 'StatusVariablesActions'], TIPSStatusVariable(fChangedVarList.Objects[i]).VariableIdent, TIPSStatusVariable(fChangedVarList.Objects[i]).VariableUseAction);
    end;

  //Save Module Settings
  if fProperties.Count > 0 then
   for i:=0 to fProperties.Count-1 do
    begin
     case TPropertyHolder(fProperties.Objects[i]).PropertyType of
      vtBoolean: fKernel.Settings.WriteInstanceBoolean(fInstanceID, ['Settings'], fProperties[i], TBoolGetter(TPropertyHolder(fProperties.Objects[i]).Getter));
      vtInteger: fKernel.Settings.WriteInstanceInteger(fInstanceID, ['Settings'], fProperties[i], TIntGetter(TPropertyHolder(fProperties.Objects[i]).Getter));
      vtFloat:   fKernel.Settings.WriteInstanceFloat(fInstanceID, ['Settings'], fProperties[i], TFloatGetter(TPropertyHolder(fProperties.Objects[i]).Getter));
      vtString:  fKernel.Settings.WriteInstanceString(fInstanceID, ['Settings'], fProperties[i], TStrGetter(TPropertyHolder(fProperties.Objects[i]).Getter));
     else
      raise EIPSModuleObject.Create(Format(_('Cannot load unsupported Property: %s'), [fProperties[i]]));
     end;
    end;
 finally
  EnableContext;
 end;

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.HasChanges: Boolean;
begin

 Result := fHasChanges;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.ResetChanges(); 
var i: Integer;
    StatusVariable: TIPSStatusVariable;
begin

 //Copy StatusVariables
 fChangedVarList.FreeObjects;
 fChangedVarList.Clear;

 if fCurrentVarList.Count > 0 then
  for i:=0 to fCurrentVarList.Count-1 do
   begin
    DisableContext;
    try
     StatusVariable := TIPSStatusVariable.Create;
    finally
     EnableContext;
    end;

    CopyObject(TIPSStatusVariable(fCurrentVarList.Objects[i]), StatusVariable);
    fChangedVarList.AddObject(StatusVariable.VariableIdent, StatusVariable);
   end;

 fHasChanges := False;
 SendSettingsChangedMessage;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.ApplyChanges();
var i: Integer;
    StatusVariable: TIPSStatusVariable;
begin

 //Save Changes
 fCurrentVarList.FreeObjects;
 fCurrentVarList.Clear;

 //Assign Variables
 AssignStatusVariables;

 if fChangedVarList.Count > 0 then
  for i:=0 to fChangedVarList.Count-1 do
   begin
    DisableContext;
    try
     StatusVariable := TIPSStatusVariable.Create;
    finally
     EnableContext;
    end;

    CopyObject(TIPSStatusVariable(fChangedVarList.Objects[i]), StatusVariable);
    fCurrentVarList.AddObject(StatusVariable.VariableIdent, StatusVariable);
   end;

 fHasChanges := False;
 SendSettingsChangedMessage;

 //FIXME send last change message

 //Force Save to XML
 SaveSettings;

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.GetInstanceID(): TInstanceID; stdcall;
begin

 Result := fInstanceID;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SettingsChanged;
begin

 fHasChanges := True;
 SendSettingsChangedMessage;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.SendSettingsChangedMessage;
begin

 if fKernelRunlevel = KR_READY then
  fKernel.PostMessage(fInstanceID, IM_CHANGESETTINGS, [fHasChanges]);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.ProcessInstanceStatusChange(InstanceID: TInstanceID; Status: Integer);
begin
end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterProperty(PropertyName: String; PType: TIPSVarType; Getter: TMethod; Setter: TMethod);
var PropHolder: TPropertyHolder;
begin

 if fProperties.IndexOf(PropertyName) <> -1 then
  EIPSModuleObject.Create(Format(_('Property "%s" already exists'), [PropertyName]));

 PropHolder := TPropertyHolder.Create;
 PropHolder.PropertyType := PType;
 PropHolder.Getter := Getter;
 PropHolder.Setter := Setter;

 fProperties.AddObject(PropertyName, PropHolder);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterFunctions();
var Prefix: String;
    Singleton: Boolean;
    i: Integer;
    Info: PTypeInfo;
    Table: PInterfaceTable;
begin

 Prefix := fKernel.ModuleLoaderEx.GetModulePrefix(GetModuleID());
 Singleton := fKernel.ModuleLoaderEx.GetModuleSingleton(GetModuleID());
 Table := Self.GetInterfaceTable;

 if Assigned(Table) then
  for i:=0 to Table.EntryCount-1 do
   begin
    Info := fKernel.ModuleLoaderEx.GetInterfaceInfo(Table.Entries[i].IID);
    if Assigned(Info) then
     begin
      if Info^.Name = 'IIPSModule' then
       fKernel.ScriptEngineEx.RegisterFunctions(Info, 'IPS', HandlePHPEvent, fInstanceID, False)
      else
       fKernel.ScriptEngineEx.RegisterFunctions(Info, Prefix, HandlePHPEvent, fInstanceID, Singleton);
     end;
   end;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterBoolProperty(PropertyName: String; Getter: TBoolGetter; Setter: TBoolSetter);
begin
 RegisterProperty(PropertyName, vtBoolean, TMethod(Getter), TMethod(Setter));
end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterIntProperty(PropertyName: String; Getter: TIntGetter; Setter: TIntSetter);
begin
 RegisterProperty(PropertyName, vtInteger, TMethod(Getter), TMethod(Setter));
end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterFloatProperty(PropertyName: String; Getter: TFloatGetter; Setter: TFloatSetter);
begin
 RegisterProperty(PropertyName, vtFloat, TMethod(Getter), TMethod(Setter));
end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterStrProperty(PropertyName: String; Getter: TStrGetter; Setter: TStrSetter);
begin
 RegisterProperty(PropertyName, vtString, TMethod(Getter), TMethod(Setter));
end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterActionHandler(VarIdent: String; Handler: TIPSActionRequestHandler);
var VarIndex: Integer;
begin

 VarIndex := fChangedVarList.IndexOf(VarIdent);

 if VarIndex = -1 then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" does not exist'), [VarIdent]));

 TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).ActionHandler := Handler;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterProfile(VarProfile: TIPSVarProfile);
begin

 //system profiles are read-only
 VarProfile.IsReadOnly := True;

 //system profile have a prefix
 if Length(VarProfile.ProfileName) > 0 then
  if VarProfile.ProfileName[1] <> '~' then
   VarProfile.ProfileName := '~' + VarProfile.ProfileName;

 //Register Profile
 fKernel.VariableManagerEx.CreateVariableProfileEx(VarProfile);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.RegisterVariable(VarIdent: String; VarName: String; VarType: TIPSVarType; VarProfile: String = ''; VarPosition: Integer = 0);
var StatusVariable: TIPSStatusVariable;
begin

 if StatusVariableExists(VarIdent) then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" already exists'), [VarIdent]));

 DisableContext;
 try
  StatusVariable := TIPSStatusVariable.Create;
 finally
  EnableContext;
 end;
 StatusVariable.VariableIdent := VarIdent;
 StatusVariable.VariableName := VarName;
 StatusVariable.VariableType := VarType;
 StatusVariable.VariablePosition := VarPosition;
 StatusVariable.VariableUseAction := True;

 //Add system profile prefix
 if Length(VarProfile) > 0 then
  if VarProfile[1] <> '~' then
   VarProfile := '~' + VarProfile;

 StatusVariable.VariableProfile := VarProfile;

 fChangedVarList.AddObject(VarIdent, StatusVariable);

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.UnregisterVariable(VarIdent: String; DeleteVariable: Boolean = False);
var VarIndex, VarID: Integer;
begin

 VarIndex := fChangedVarList.IndexOf(VarIdent);

 if VarIndex = -1 then
  raise EIPSModuleObject.Create(Format(_('StatusVariable "%s" does not exist'), [VarIdent]));

 try
  if DeleteVariable then
   begin
    VarID := TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).VariableID;
    if fKernel.VariableManager.VariableExists(VarID) then
     fKernel.VariableManager.DeleteVariable(VarID);
   end;

  fChangedVarList.Objects[VarIndex].Free;
  fChangedVarList.Delete(VarIndex);
 except
  LogMessage(KL_WARNING, Format(_('Could not unregister Variable: %s'), [VarIdent]));
 end;

end;

//------------------------------------------------------------------------------
procedure TIPSModuleObject.MaintainVariable(VarIdent: String; VarName: String; VarType: TIPSVarType; Keep: Boolean; VarProfile: String = ''; VarPosition: Integer = 0);
var VariableID: TVariableID;
    Variable: TIPSVariable;
    VarIndex: Integer;
begin

 //Add system profile prefix
 if Length(VarProfile) > 0 then
  if VarProfile[1] <> '~' then
   VarProfile := '~' + VarProfile;

 if StatusVariableExists(VarIdent) then
  begin
   if Keep then
    begin
     VariableID := GetStatusVariableID(VarIdent);
     if VariableID > 0 then
      if fKernel.VariableManager.VariableExists(VariableID) then
       begin
        Variable := fKernel.VariableManager.GetVariable(VariableID);
        try
         if (Variable.VariableValue.ValueType <> VarType) then
          begin
           UnregisterVariable(VarIdent, True);
           RegisterVariable(VarIdent, VarName, VarType, VarProfile, VarPosition);
          end
         else if (Variable.VariableProfile <> VarProfile) then
          begin
           VarIndex := fChangedVarList.IndexOf(VarIdent);
           TIPSStatusVariable(fChangedVarList.Objects[VarIndex]).VariableProfile := VarProfile;           
          end;
        finally
         Variable.Free;
        end;
       end;
    end
   else
    UnregisterVariable(VarIdent, True);
  end
 else
  begin
   if Keep then
    RegisterVariable(VarIdent, VarName, VarType, VarProfile, VarPosition);
  end; 

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.RequireParent(ModuleID: TGUID; ForceCreate: Boolean = False): Boolean;
var Compatibles: TInstanceIDs;
    InstanceID: TInstanceID;
    Module: TIPSModuleClass;
begin

 Result := False;
 if fKernel.DataHandler.GetInstanceParentID(fInstanceID) = 0 then
  begin
   if not ForceCreate then
    Compatibles := fKernel.InstanceManager.GetCompatibleInstances(fInstanceID);

   if ForceCreate or (High(Compatibles) = -1) then
    begin
     if not fKernel.ModuleLoader.ModuleExists(GUIDToString(ModuleID)) then
      exit;
     Module := fKernel.ModuleLoaderEx.GetModuleClass(GUIDToString(ModuleID));
     InstanceID := fKernel.InstanceManager.CreateInstance(GUIDToString(ModuleID));
     fKernel.ObjectManager.SetName(InstanceID, Module.GetModuleName());
     fKernel.DataHandler.ConnectInstance(fInstanceID, InstanceID);
     Result := True;
    end
   else
    fKernel.DataHandler.ConnectInstance(fInstanceID, Compatibles[0]);
  end;

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.ForceParent(ModuleID: TGUID): Boolean;
var ParentID: TInstanceID;
    InstanceID: TInstanceID;
    Instance: TIPSInstance;
    InstanceModuleID: TStrGUID;
    Module: TIPSModuleClass;
begin

 Result := False;

 ParentID := fKernel.DataHandler.GetInstanceParentID(fInstanceID);
 if ParentID <> 0 then
  begin
   Instance := fKernel.InstanceManager.GetInstance(ParentID);
   try
    InstanceModuleID := Instance.ModuleInfo.ModuleID;
   finally
    Instance.Free;
   end;
   if not (InstanceModuleID = GUIDToString(ModuleID)) then
    if fKernel.ModuleLoader.ModuleExists(GUIDToString(ModuleID)) then
     begin
      Module := fKernel.ModuleLoaderEx.GetModuleClass(GUIDToString(ModuleID));
      InstanceID := fKernel.InstanceManager.CreateInstance(GUIDToString(ModuleID));
      fKernel.ObjectManager.SetName(InstanceID, Module.GetModuleName());
      fKernel.DataHandler.ConnectInstance(fInstanceID, InstanceID);

      //Delete old Parent?
      if not fKernel.DataHandler.HasInstanceChildren(ParentID) then
       fKernel.InstanceManager.DeleteInstance(ParentID);

      Result := True;
     end;
  end;

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.GetParent(): IIPSModule;
var ObjID: TInstanceID;
    Instance: TIPSInstance;
begin

 ObjID := fKernel.DataHandler.GetInstanceParentID(fInstanceID);
 if ObjID = 0 then
  raise EIPSModuleObject.Create(_('Instance has no Parent Instance connected!'));

 Instance := fKernel.InstanceManager.GetInstance(ObjID);
 try
  Result := Instance.InstanceInterface;
 finally
  Instance.Free;
 end;

 if Result = nil then
  raise EIPSModuleObject.Create(_('Instance Interface is not assigned!'));

end;

//------------------------------------------------------------------------------
function TIPSModuleObject.HasActiveParent(): Boolean;

 function HasActiveParentByID(InstanceID: TInstanceID): Boolean;
 var ObjID: TInstanceID;
     Instance: TIPSInstance;
 begin

  Result := False;
  ObjID := fKernel.DataHandler.GetInstanceParentID(InstanceID);
  if ObjID > 0 then
   begin
    Instance := fKernel.InstanceManager.GetInstance(ObjID);
    try
     if Instance.ModuleInfo.ModuleType <> mtIO then
      Result := HasActiveParentByID(Instance.InstanceID)
     else
      Result := Instance.InstanceStatus = IS_ACTIVE;
    finally
     Instance.Free;
    end;
   end;
   
 end;

begin
 Result := HasActiveParentByID(fInstanceID);
end;

//------------------------------------------------------------------------------
function TIPSModuleObject.GetChildren(): TInterfaceList;
var ObjIDs: TInstanceIDs;
    i: Integer;
    Instance: TIPSInstance;
begin

 ObjIDs := fKernel.DataHandler.GetInstanceChildrenIDs(fInstanceID);

 Result := TInterfaceList.Create;
 for i:= Low(ObjIDs) to High(ObjIDs) do
  if fKernel.InstanceManager.InstanceExists(ObjIDs[i]) then
   begin
    Instance := fKernel.InstanceManager.GetInstance(ObjIDs[i]);
    try
     if Assigned(Instance.InstanceInterface) then
      Result.Add(Instance.InstanceInterface);
    finally
     Instance.Free;
    end;
   end;

end;

//------------------------------------------------------------------------------
class function TIPSModuleObject.GetParentRequirements(): TStrGUIDs;
begin
 if GetModuleType = mtIO then
  raise EIPSModuleObject.Create(_('I/O Modules have no Parents'));
 SetLength(Result, 0);
end;

//------------------------------------------------------------------------------
class function TIPSModuleObject.GetChildRequirements(): TStrGUIDs;
begin
 if GetModuleType in [mtConfigurator, mtCore, mtDevice] then
  raise EIPSModuleObject.Create(_('Device/Core/Configurator Modules have no Children'));
 SetLength(Result, 0);
end;

//------------------------------------------------------------------------------
class function TIPSModuleObject.GetImplemented(): TStrGUIDs;
begin
 SetLength(Result, 0);
end;

//------------------------------------------------------------------------------
class function TIPSModuleObject.GetVendor(): String;
begin
 Result := '';
end;

//------------------------------------------------------------------------------
class function TIPSModuleObject.GetAliases(): TStringArray;
begin
 SetLength(Result, 0);
end;

//Implements some global used functions/procedures
//------------------------------------------------------------------------------
function StringToVarType(VarType: String): TIPSVarType;
begin
 Result := TIPSVarType(GetEnumValue(TypeInfo(TIPSVarType),'vt'+VarType));
end;

//------------------------------------------------------------------------------
function VarTypeToString(VarType: TIPSVarType): String;
begin
 Result := GetEnumName(TypeInfo(TIPSVarType), Integer(VarType));
 Delete(Result, 1, 2); //Cut off first two chars
end;

//------------------------------------------------------------------------------
function StringToEventType(EventType: String): TIPSEventType;
begin
 Result := TIPSEventType(GetEnumValue(TypeInfo(TIPSEventType),'et'+EventType));
end;

//------------------------------------------------------------------------------
function EventTypeToString(EventType: TIPSEventType): String;
begin
 Result := GetEnumName(TypeInfo(TIPSEventType), Integer(EventType));
 Delete(Result, 1, 2); //Cut off first two chars
end;

//------------------------------------------------------------------------------
function StringToTriggerType(TriggerType: String): TIPSTriggerType;
begin
 Result := TIPSTriggerType(GetEnumValue(TypeInfo(TIPSTriggerType),'evt'+TriggerType));
end;

//------------------------------------------------------------------------------
function TriggerTypeToString(TriggerType: TIPSTriggerType): String;
begin
 Result := GetEnumName(TypeInfo(TIPSTriggerType), Integer(TriggerType));
 Delete(Result, 1, 3); //Cut off first three chars
end;

//------------------------------------------------------------------------------
function StringToCyclicDateType(CyclicDateType: String): TIPSCyclicDateType;
begin
 Result := TIPSCyclicDateType(GetEnumValue(TypeInfo(TIPSCyclicDateType),'cdt'+CyclicDateType));
end;

//------------------------------------------------------------------------------
function CyclicDateTypeToString(CyclicDateType: TIPSCyclicDateType): String;
begin
 Result := GetEnumName(TypeInfo(TIPSCyclicDateType), Integer(CyclicDateType));
 Delete(Result, 1, 3); //Cut off first three chars
end;

//------------------------------------------------------------------------------
function StringToCyclicTimeType(CyclicTimeType: String): TIPSCyclicTimeType;
begin
 Result := TIPSCyclicTimeType(GetEnumValue(TypeInfo(TIPSCyclicTimeType),'cdt'+CyclicTimeType));
end;

//------------------------------------------------------------------------------
function CyclicTimeTypeToString(CyclicTimeType: TIPSCyclicTimeType): String;
begin
 Result := GetEnumName(TypeInfo(TIPSCyclicTimeType), Integer(CyclicTimeType));
 Delete(Result, 1, 3); //Cut off first three chars
end;

initialization
 RemClassRegistry.RegisterXSInfo(TypeInfo(TIPSID), 'urn:UIPSTypes', 'TIPSID');
 RemClassRegistry.RegisterXSInfo(TypeInfo(TInstanceID), 'urn:UIPSTypes', 'TInstanceID');

end.




